//Ŀ
//                                                                       
//  MODULE NAME      CA83.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME ECI Samples for the CICS Gateway for Java           
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA83 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
// ascj001 - Multiple Asynchronous ECI Calls with Callback
//
// Written : May 97 - Alan Hollingshead
// Updated : Jul 97 - default application now SRVTIME, not FAADECIC
//
// This applet/application can run multiple ECI asynchronous calls with
// callback simultaneously.
//
// This source file creates two classes:
// (1) ascj001.class ........ issues the asynchronous calls.
// (2) TestCB.class ......... handles all callbacks received.
//
// Issuing "java ascj001 -?" shows the following help and defaults:
//
//       ASCJ001 - Multiple Asynchronous ECI Calls with Callback
//       Usage: java ascj001 <parameters>
//              -j  JGate              Default: (local hostname)
//              -r  JGate port         Default: 2006
//              -s  CICS server        Default:
//              -a  application        Default: SRVTIME
//              -c  # calls            Default: 1
//              -i  # iterations       Default: 1
//              -d  delay              Default: 0 seconds
//              -u  userid             Default: SYSAD
//              -p  password           Default: SYSAD
//              -!  debug              Default: false
//              -?  This help panel
//
// Example: java ascj001 -s cicstcp
//
// The above example overrides the server setting of <null> with cicstcp.
//
// Parameter overrides:
//      -j ... hostname or IP address where Java Gateway is running.
//             The default is your local hostname.
//      -p ... Java Gateway port. This defaults to 2006 unless
//             otherwise specified when starting the Java Gateway.
//      -s ... if the CICS Client is being used to connect to a CICS
//             server, null takes the first CICSCLI.INI setting.
//             Otherwise specify a valid entry defined in the CICSCLI.INI
//             file. If a CICS Client is not being used, a null value
//             will use the local server ECI, if supported.
//      -a ... name of server application.
//      -c ... number of calls within an iteration. Each call is assigned
//             a different message_qualifier.
//      -i ... number of iterations.
//      -d ... a delay in seconds between each call (as specified by -c).
//      -u ... userid, if required by the server mirror transaction.
//      -p ... password, if required by the server mirror transaction.
//      -! ... some debug info is supplied if "-!" is specified.
//      -? ... displays the help panel and defaults.
//
// Prior to each ECI call the commarea is initialised to
// 18 dashes. The server returns the current date and timestamp.
// The default CICS server program is the supplied SRVTIME
// which is compatible on all CICS servers. If desired, a different server
// program can be supplied as an input parameter to this applet/application.
// This program contains no data conversion for the commarea.
//
// If run as an applet - parameters are passed via ascj001.html.
// If run as an application - parameters can be overridden via command line.
//
//
import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.applet.*;

public class ascj001 extends Applet implements Runnable {
   // set up variables
   int iEci_call, iNum_Calls, iCounter, iIt_Count, iReply_Count;
   int     iCalls       = 1;
   int     iIterations  = 1;
   int     iPort        = 2006;
   int     iDelay       = 0;
   int     iCallsIssued = 0;
   static int iReplyOK  = 0;
   String  sDebug       = " ";
   String  sWait        = " ";
   String  sGetReply    = "";
   boolean bReply_Rcvd  = false;

   // CICS commarea
   byte[] Commarea = new byte[18];

   // application defaults
   String sServer      = "" ;
   String sUseServer   = "" ;
   String sUserid      = "SYSAD" ;
   String sPassword    = "SYSAD" ;
   String sProgname    = "SRVTIME" ;
   String sTransid     = "CPMI" ;
   boolean bDebug      = false ;
   boolean bWait       = false ;
   String sJGate       = "" ;
   String sPort        = "2006" ;
   String sCalls       = "1";
   String sIterations  = "1";
   String sDelay       = "0";

   // Text area for program output
   TextArea texMessages;
   TextArea texSettings;

   // callback variables
   int    iMessage_Qualifier   = 99;
   Object ObjLock              = null;
   static int iFlag            = 0;

   // ------------------------------------------------------------------
   // Methods: displaySettings and displayMsg
   // When running as an applet ........ output sent to TextArea
   // When running as an application ... output sent to console
   // ------------------------------------------------------------------
   public void displaySettings(String strMessage) {
      if (texSettings != null) {
         texSettings.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }
   public void displayMsg(String strMessage) {
      if (texMessages != null) {
         texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }

   // ------------------------------------------------------------------
   // Method: init
   // Reads relevant parameters from html file when running as an applet
   // ------------------------------------------------------------------
   public void init() {
      // read applet parameters from html file
      sJGate      = getParameter("j");
      sPort       = getParameter("r");
      sServer     = getParameter("s");
      sProgname   = getParameter("a");
      sCalls      = getParameter("c");
      iCalls      = Integer.parseInt(sCalls);
      sIterations = getParameter("i");
      iIterations = Integer.parseInt(sIterations);
      sUserid     = getParameter("u");
      sPassword   = getParameter("p");
      sDelay      = getParameter("d");
      // convert to integer, and multiply to make seconds
      iDelay      = Integer.parseInt(sDelay) * 1000;
      sDebug      = getParameter("!");
      // convert to boolean
      if (sDebug.equals("true")) bDebug = true;
      if (sWait.equals("true"))  bWait  = true;

      // Create a text area to display messages
      setLayout(null);
      Color lightYellow = new Color(240,240,204);
      setBackground(lightYellow);

      // add textareas
      texSettings = new TextArea();
      texMessages = new TextArea();
      add(texMessages);
      add(texSettings);

      // distance from left, top, then width, height
      texSettings.reshape(5, 130, 430, 290);
      texMessages.reshape(435, 30, 550, 400);

      // define font and colours
      texSettings.setFont(new Font("Courier", Font.PLAIN, 11));
      texSettings.setEditable(false);
      texSettings.setForeground(Color.darkGray);
      texSettings.setBackground(Color.white);

      texMessages.setFont(new Font("Courier", Font.PLAIN, 11));
      texMessages.setEditable(false);
      texMessages.setForeground(Color.blue);
      texMessages.setBackground(Color.white);

      Thread test = new Thread(this);
      this.show();
      this.run();
   }

   // ------------------------------------------------------------------
   // Method: paint
   // Display headings above the textarea
   // ------------------------------------------------------------------
   public void paint(Graphics g) {
      // simple heading
      g.setFont(new Font("Helvetica", Font.BOLD, 14));
      Color purple = new Color(39,25,204);
      g.setColor(purple);
      // distance from left, top
      g.drawString("ASCJ001", 10, 60);
      g.drawString("Multiple Asynchronous ECI Calls with Callback", 10, 90);
   }

   // ------------------------------------------------------------------
   // Method: main
   // read in parameters when run as an application
   // ------------------------------------------------------------------
   static public void main(String[] args) {
      // declare and instantiate testApp object based on class ascj001
      ascj001 testApp = new ascj001();

      // parse arguments entered on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            if (args[loop].indexOf("-!") > -1) {
               // set debug ON
               testApp.bDebug=true ;
               // for debug purposes
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            else if (args[loop].indexOf("-s") > -1) {
               // set cics server
               testApp.sServer = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-j") > -1) {
               // set jgate server
               testApp.sJGate = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-r") > -1) {
               // set jgate port
               testApp.sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iPort = Integer.parseInt(testApp.sPort);
            }
            else if (args[loop].indexOf("-u") > -1) {
               // set userid
               testApp.sUserid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-p") > -1) {
               // set password
               testApp.sPassword = args[loop+1] ;
                ++loop ;
            }
            else if (args[loop].indexOf("-a") > -1) {
               // set progname
               testApp.sProgname = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-c") > -1) {
               // set Calls
               testApp.sCalls = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iCalls = Integer.parseInt(testApp.sCalls);
            }
            else if (args[loop].indexOf("-i") > -1) {
               // set Iterations
               testApp.sIterations = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iIterations = Integer.parseInt(testApp.sIterations);
            }
            else if (args[loop].indexOf("-d") > -1) {
               // set Delay between each ECI call
               testApp.sDelay = args[loop+1] ;
               ++loop ;
               // convert to integer, and multiply to make seconds
               testApp.iDelay = Integer.parseInt(testApp.sDelay) * 1000;
            }
            else if ((args[loop].indexOf("-?") > -1) ||
                     (args[loop].indexOf("help") > -1)) {
               // HELP
               testApp.displaySettings("\nASCJ001 - Multiple Asynchronous ECI Calls with Callback") ;
               testApp.displaySettings("Usage: java ascj001 <parameters>");
               testApp.displaySettings("       -j  JGate                Default: (local hostname)");
               testApp.displaySettings("       -r  JGate port           Default: "+testApp.sPort);
               testApp.displaySettings("       -s  CICS server          Default: (not specified)");
               testApp.displaySettings("       -a  application          Default: "+testApp.sProgname);
               testApp.displaySettings("       -c  # calls              Default: "+testApp.sCalls);
               testApp.displaySettings("       -i  # iterations         Default: "+testApp.sIterations);
               testApp.displaySettings("       -d  delay between calls  Default: "+testApp.sDelay+" seconds");
               testApp.displaySettings("       -u  userid               Default: "+testApp.sUserid);
               testApp.displaySettings("       -p  password             Default: "+testApp.sPassword);
               testApp.displaySettings("       -!  debug                Default: "+testApp.bDebug);
               testApp.displaySettings("       -?  This help panel");
               return;
            }
         // end FOR loop
         }
      // end-if-args
      }
   // now go and do the work
   testApp.run();
   // end-main
   }

   // ------------------------------------------------------------------------
   // Method: run
   // Real work starts here
   // ------------------------------------------------------------------------
   public void run() {

      // if JGate not specified then use local hostname.
      // may require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            displaySettings("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

      // check if server specified. If not, server ECI can be used if on
      // same machine as gateway, and server ECI module is earlier in LIBPATH
      // than a CICS Client one. Else use first server in Client CICSCLI.INI.
      if (sServer.equals("")) {
         sUseServer = "(not specified)";
      }
      else {
         sUseServer = sServer;
      }
      displaySettings("\nASCJ001 - Multiple Asynchronous ECI Calls with Callback") ;
      displaySettings("       -j  JGate                -> "+sJGate);
      displaySettings("       -r  JGate port           -> "+sPort);
      displaySettings("       -s  CICS server          -> "+sUseServer);
      displaySettings("       -a  application          -> "+sProgname);
      displaySettings("       -c  # calls              -> "+sCalls);
      displaySettings("       -i  # iterations         -> "+sIterations);
      displaySettings("       -d  delay between calls  -> "+sDelay+" seconds");
      displaySettings("       -u  userid               -> "+sUserid);
      displaySettings("       -p  password             -> "+sPassword);
      displaySettings("       -!  debug                -> "+bDebug);
      displaySettings("       -?  help panel");

      // create connection to JGate
      JavaGateway jgaConnection;
      jgaConnection=null;
      try {
         jgaConnection = new JavaGateway(sJGate, iPort);
         displayMsg("\n--- Successfully created JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Failed to create JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
         return;
      }

      // if server name has not been provided, get the server name
      if (sServer.equals("")) {
         ECIRequest eciRequest = ECIRequest.listSystems(1);
         try {
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            displayMsg("\n--- ERROR during flow of listSystems(1) request ---");
            return;
         }
         // output info on the Java Vector
         displaySettings("\n--- Using CICS Region: "+ eciRequest.SystemList.elementAt(0));
      }

      // output first iteration banner
      displayMsg("\n#   Call               Commarea Data      Length CICS Rc  Abend  Call OK?");
      displayMsg("--  -----------------  -----------------  ------ -------  -----  --------");

      iteration_loop:
      // Iteration loop (-i parameter)
      for (iIt_Count=1; iIt_Count<=iIterations; iIt_Count++) {

         // output banner every 5 iterations
         if (iIt_Count%5==0) {
            displayMsg("\n#   Call               Commarea Data      Length CICS Rc  Abend  Call OK?");
            displayMsg("--  -----------------  -----------------  ------ -------  -----  --------");
         } // end-if

         // declare and instantiate object for lock
         ObjLock         = new Object();

         Thread thrMain  = Thread.currentThread();

         // Calls loop (-c parameter)
         for (iNum_Calls=1; iNum_Calls < (iCalls+1); iNum_Calls++) {

            // new commarea required for each call
            byte[] Commarea = new byte[18];
            try {

               // new calBack for each call
               TestCB calBack  = new TestCB(thrMain, ObjLock, iNum_Calls, texMessages);
               ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_ASYNC,
                                                        sServer,
                                                        sUserid,
                                                        sPassword,
                                                        sProgname,
                                                        sTransid,
                                                        Commarea,
                                                        0,
                                                        ECIRequest.ECI_NO_EXTEND,
                                                        ECIRequest.ECI_LUW_NEW,
                                                        iMessage_Qualifier,
                                                        calBack
                                                        );

               // set commarea to dashes
               for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
               // reset commarea to dashes
               eciRequest.Commarea[iCounter]=45;
               // Convert ASCII bytes to ASCII characters
               String sDashData  = new String(eciRequest.Commarea,0);

               eciRequest.Message_Qualifier = iNum_Calls;

               if (iIt_Count < 10) {
                  displayMsg(iIt_Count+"   eci_async + mq "
                              + eciRequest.Message_Qualifier + "    "
                              + "  -- flow --> ");
               }
               else {
                  displayMsg(iIt_Count+"  eci_async + mq "
                              + eciRequest.Message_Qualifier + "    "
                              + " -- flow! --> ");
               }
               // flow ECI request
               iCallsIssued++;
               jgaConnection.flow(eciRequest);

               // Delay next ECI call if delay parameter specified (default=0)
               try {
                  Thread.sleep(iDelay);
               }
               catch (Exception e) {
                  GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                  displayMsg("\n--- Error during sleep() ---\n");
                  displayMsg("--- Caught Exception: " + e);
                  break iteration_loop;
               }
            }
            catch (Exception e) {
               GatewayTrace.traceln("ERROR: Caught Exception: " + e);
               displayMsg("\n--- Error returned by JGate ---\n");
               displayMsg("--- Caught Exception: " + e);
               break iteration_loop;
            }
         }
         // this will synchronize the last object in the iteration
         synchronized(ObjLock)
         {
            while (iFlag != (iCalls * iIt_Count))
            {
               try
               {
             //    displayMsg("About to wait() for "+ObjLock);
             //    displayMsg("wait iFlag = "+iFlag);
                   ObjLock.wait();
             //    displayMsg("woken");
               }
               catch (Exception e) {
                  GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                  displayMsg("\n--- Error returned by JGate ---\n");
                  displayMsg("--- Caught Exception: " + e);
                  break iteration_loop;
               }
            }
         }
         //   displayMsg("Back from wait()");
      } //end iIt_Count

      // ------------------------------------------------------------------
      // Close connection to JGate
      // ------------------------------------------------------------------
      try {
         jgaConnection.close();
         displayMsg("\n--- Successfully closed JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Error closing JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
      }

      displaySettings("\n--- Calls successful ... "+
                 +iReplyOK + " out of " + iCallsIssued );
      displaySettings("--- Iterations run ..... "+ (iIt_Count-1)
                           + " out of " + iIterations );

   } // end-run() method

} // end-ascj001 class

// ------------------------------------------------------------------------
// Class: TestCB (ie TestCallback)
// Handles all callbacks received from the Java gateway.
// ------------------------------------------------------------------------
class TestCB implements Callbackable
{
   ECIRequest CB_eciRequest  = null;
   Thread thrToWakeUp        = null;
   Object    ObjLock         = null;
   boolean boolIsAlive       = false;
   int myCallNo              = 0;
   String sMQok              = "OK";
   TextArea CB_texMessages   = null;

   public TestCB(Thread thrWakeUp, Object ObjParm, int iCallNo, TextArea texMessages)
   {
       thrToWakeUp    = thrWakeUp;
       ObjLock        = ObjParm;
       myCallNo       = iCallNo;
       CB_texMessages = texMessages;
   }

   public void displayMsg(String strMessage) {
      if (CB_texMessages != null) {
         CB_texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }

   public void setResults(GatewayRequest gatReply)
   {
      //  displayMsg("setResults()"+myCallNo);
      CB_eciRequest  = (ECIRequest) gatReply;
   }

   public void run()
   {
      //  displayMsg("Callback is running");
      //  displayMsg("Message_Qualifier = " + CB_eciRequest.Message_Qualifier);
      //  displayMsg("Cics_Rc = " + CB_eciRequest.Cics_Rc);

      boolIsAlive = thrToWakeUp.isAlive();

      if (boolIsAlive) {
         //  displayMsg("Rousing main thread ...");

         synchronized (ObjLock) {
            ascj001.iFlag++;
            //     displayMsg("iFlag = " + ascj001.iFlag);
            //     displayMsg("Notifying : "+ObjLock);
            ObjLock.notify();

            String ComData;
            // Convert ASCII bytes to ASCII characters
            ComData = new String(CB_eciRequest.Commarea,0);

            // confirm myCallNo, which assigns Message_Qualifier in initial
            // async call, matches returned Message_Qualifier in eciRequest
            // response.
            if (myCallNo == CB_eciRequest.Message_Qualifier) {
               sMQok = " OK";
            }
            else {
               sMQok = "Wrong MQ!";
            }

            // confirm commarea has been modified by server program
            // otherwise call was not successful.
            if (ComData.startsWith("---")) {
               sMQok = "Commarea incorrect";
            }
            else {
               sMQok = " OK";
            }

            // check CICS Rc
            if (CB_eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
               sMQok = " OK";
               // keep tally of successful calls
               ascj001.iReplyOK++;
            }
            else {
               sMQok = "Call failed";
            }

            // Display result
            displayMsg("    eci_async + mq "
                        + CB_eciRequest.Message_Qualifier + "   " +
                        ComData.substring(0,CB_eciRequest.Commarea_Length-1) +
                        "  " + CB_eciRequest.Commarea_Length +"     "+
                        CB_eciRequest.Cics_Rc + "       " +
                        CB_eciRequest.Abend_Code + "   "+sMQok);
         }
      }
      else
      {
         // big trouble as main thread has disappeared!
         displayMsg("Main thread is dead ...");
      }
   }
}
