//Ŀ
//                                                                       
//  MODULE NAME      CA83.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME ECI Samples for the CICS Gateway for Java           
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA83 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
// asmj002 - Asynchronous Calls with Message Qualifiers.
//           Followed by:
//           ECI_GET_SPECIFIC_REPLY_WAIT or ECI_GET_SPECIFIC_REPLY loop.
//
// Written : Apr 97 - Alan Hollingshead
// Updated : Jul 97 - default application now SRVTIME, not FAADECIC
//
// Summary : Issue 5 async calls with Message Qualifiers  5, 4, 3, 2, 1,
//           with optional 2 second delay between each one.
//           Issue ECI_GET_SPECIFIC_REPLY(_WAIT) for each Message_Qualifier
//           in order 1, 2, 3, 4, 5 checking timestamps are in reverse order.
//
// asmj002 is a Java program that on each requested iteration
// issues five asynchronous calls with Message Qualifiers assigned
// 5 through 1 (in reverse order).
// This is followed either by an ECI_GET_SPECIFIC_REPLY_WAIT (if the -w
// option is specified), or an ECI_GET_SPECIFIC_REPLY (the default) for
// each Message Qualifier in the ascending order 1 through 5.
// The timestamps returned in the commarea should appear in descending
// order to indicate that the eci_get_specific_reply(_wait) picked
// up the related Message Qualifier. On fast networks the timestamps
// may appear the same, and so the -2 option can be specified to
// separate each asynchronous call by 2 seconds (see Options: below).
//
// ECI_GET_SPECIFIC_REPLY calls:
// Up to nine ECI_GET_SPECIFIC_REPLY calls are issued to obtain
// the required response from the server. Between each attempt to
// receive this reply there is a one second delay to allow for network
// traffic or a busy server. The output shows which call obtained the
// response. For example, "eci_get_specific_reply #3 MQ 5" means that
// the response was obtained on the third attempt for Message Qualifier 5.
// If the result shows "eci_get_specific_reply #9" with a CICS Rc of -5,
// this indicates that the response was not received after nine attempts.
//
// Prior to each asynchronous call the commarea is initialised to
// 18 dashes. The server returns the current date and timestamp.
// The default CICS server program is the supplied SRVTIME
// which is compatible on all CICS servers. If desired, a different server
// program can be supplied as an input parameter to this applet/application.
//
// After all iterations have run, a banner displays the number of successful
// calls within iterations where asynchronous calls returned positive replies.
//
// Options: -w ... eci_get_specific_reply is the default. The -w option
//                 requests eci_get_specific_reply_wait.
//          -2 ... between each asynchronous call a 2 second delay is
//                 issued. This allows the user to see the timestamps
//                 on the replies in reverse order to confirm that the
//                 eci Message_Qualifier is being used correctly.
//          -c ... in certain circumstances (eg timeouts, or security
//                 error received on eci_get_specific_reply(_wait))
//                 there may be outstanding replies from previous runs.
//                 The -c option issues repeated eci_get_specific_reply
//                 calls until eci_err_no_reply reports no more outstanding
//                 replies. (Alternatively stop and restart JGate).
//                 Security errors are specifically ignored.
//
// Warning: using the -c and -w options
//          Care should be taken when running the program from multiple
//          processes as the -c option may clear outstanding replies
//          which another process is waiting on (if -w specified).
//
// Data conversion:
//          This program contains no data conversion for the commarea.
//          Any data conversion should be carried out on the server.
//
// If run as an applet - parameters are passed via asmj002.html.
// If run as an application - parameters can be overridden via command line.
// Issue "asmj002 -?" for help and the default settings.
//
//
import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.applet.*;

public class asmj002 extends Applet implements Runnable {
   // set up variables
   int iEci_Call, iLUW, iCounter, iIt_Count, iReply_Count;
   int iEci_Count, iMsg_Qual;
   int     iIterations   = 1;
   int     iPort         = 2006;
   int     iDelay        = 0;
   int     iReplyOK      = 0;
   int     iCallsIssued  = 0;
   String  sDebug        = " ";
   String  sWait         = " ";
   String  sGetReply     = "";
   String  sAsyncDelay   = "";
   String  sClearReplies = "";
   boolean bReply_Rcvd   = false;

   // CICS commarea
   byte[] Commarea = new byte[18];

   // application defaults
   String sServer      = "" ;
   String sUseServer   = "" ;
   String sUserid      = "SYSAD" ;
   String sPassword    = "SYSAD" ;
   String sProgname    = "SRVTIME" ;
   String sTransid     = "CPMI" ;
   boolean bDebug      = false ;
   boolean bWait       = false ;
   boolean bAsyncDelay = false ;
   boolean bClearReplies = false ;
   String sJGate       = "" ;
   String sPort        = "2006" ;
   String sIterations  = "1";
   String sDelay       = "0";

   // Text area for program output
   TextArea texMessages;
   TextArea texSettings;

   // ------------------------------------------------------------------
   // Methods: displaySettings and displayMsg
   // When running as an applet ........ output sent to TextArea
   // When running as an application ... output sent to console
   // ------------------------------------------------------------------
   public void displaySettings(String strMessage) {
      if (texSettings != null) {
         texSettings.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }
   public void displayMsg(String strMessage) {
      if (texMessages != null) {
         texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }

   // ------------------------------------------------------------------
   // Method: init
   // Reads relevant parameters from html file when running as an applet
   // ------------------------------------------------------------------
   public void init() {
      // read applet parameters from html file
      sJGate        = getParameter("j");
      sPort         = getParameter("r");
      sServer       = getParameter("s");
      sProgname     = getParameter("a");
      // sNum_LUWs   = getParameter("l");
      // iNum_LUWs   = Integer.parseInt(sNum_LUWs);
      sIterations   = getParameter("i");
      iIterations   = Integer.parseInt(sIterations);
      sUserid       = getParameter("u");
      sPassword     = getParameter("p");
      sDelay        = getParameter("d");
      // convert to integer, and multiply to make seconds
      iDelay        = Integer.parseInt(sDelay) * 1000;
      sWait         = getParameter("w");
      sAsyncDelay   = getParameter("2");
      sClearReplies = getParameter("e");
      sDebug        = getParameter("!");
      // convert to boolean
      if (sWait.equals("true"))  bWait  = true;
      if (sDebug.equals("true")) bDebug = true;
      if (sAsyncDelay.equals("true")) bAsyncDelay = true;

      // Create a text area to display messages
      setLayout(null);
      Color lightYellow = new Color(240,240,204);
      setBackground(lightYellow);

      // add textareas
      texSettings = new TextArea();
      texMessages = new TextArea();
      add(texMessages);
      add(texSettings);

      // distance from left, top, then width, height
      texSettings.reshape(5, 130, 430, 290);
      texMessages.reshape(435, 30, 550, 400);

      // define font and colours
      texSettings.setFont(new Font("Courier", Font.PLAIN, 11));
      texSettings.setEditable(false);
      texSettings.setForeground(Color.darkGray);
      texSettings.setBackground(Color.white);

      texMessages.setFont(new Font("Courier", Font.PLAIN, 11));
      texMessages.setEditable(false);
      texMessages.setForeground(Color.blue);
      texMessages.setBackground(Color.white);

      Thread test = new Thread(this);
      this.show();
      this.run();
   }

   // ------------------------------------------------------------------
   // Method: paint
   // Display the output
   // ------------------------------------------------------------------
   public void paint(Graphics g) {
      // simple heading
      g.setFont(new Font("Helvetica", Font.BOLD, 14));
      Color purple = new Color(39,25,204);
      g.setColor(purple);
      // distance from left, top
      g.drawString("ASMJ002", 10, 50);
      g.drawString("ECI Asynchronous Calls", 10, 80);
      g.drawString("Incorporating Message Qualifiers", 10, 110);
   }

   // ------------------------------------------------------------------
   // Method: main
   // read in parameters when run as an application
   // ------------------------------------------------------------------
   static public void main(String[] args) {
      // declare and instantiate testApp object based on class asmj002
      asmj002 testApp = new asmj002();

      // parse arguments entered on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            if (args[loop].indexOf("-!") > -1) {
               // set debug ON
               testApp.bDebug=true ;
               // for debug purposes
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            if (args[loop].indexOf("-w") > -1) {
               // use eci_get_reply_wait
               testApp.bWait=true ;
            }
            if (args[loop].indexOf("-2") > -1) {
               // 2 sec delay between async calls
               testApp.bAsyncDelay=true ;
            }
            if (args[loop].indexOf("-c") > -1) {
               // clear outstanding replies
               testApp.bClearReplies=true ;
            }
            else if (args[loop].indexOf("-s") > -1) {
               // set cics server
               testApp.sServer = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-j") > -1) {
               // set jgate server
               testApp.sJGate = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-r") > -1) {
               // set jgate port
               testApp.sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iPort = Integer.parseInt(testApp.sPort);
            }
            else if (args[loop].indexOf("-u") > -1) {
               // set userid
               testApp.sUserid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-p") > -1) {
               // set password
               testApp.sPassword = args[loop+1] ;
                ++loop ;
            }
            else if (args[loop].indexOf("-a") > -1) {
               // set progname
               testApp.sProgname = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-i") > -1) {
               // set Iterations
               testApp.sIterations = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iIterations = Integer.parseInt(testApp.sIterations);
            }
            else if (args[loop].indexOf("-d") > -1) {
               // set Delay between each ECI call
               testApp.sDelay = args[loop+1] ;
               ++loop ;
               // convert to integer, and multiply to make seconds
               testApp.iDelay = Integer.parseInt(testApp.sDelay) * 1000;
            }
            else if ((args[loop].indexOf("-?") > -1) ||
                     (args[loop].indexOf("help") > -1)) {
               // HELP
               testApp.displaySettings("\nASMJ002 - ECI Asynchronous Calls with Message Qualifiers") ;
               testApp.displaySettings("Usage: java asmj002 <parameters>");
               testApp.displaySettings("       -j  JGate                             Default: (local hostname)");
               testApp.displaySettings("       -r  JGate port                        Default: "+testApp.sPort);
               testApp.displaySettings("       -s  CICS server                       Default: (not specified)");
               testApp.displaySettings("       -a  application                       Default: "+testApp.sProgname);
               testApp.displaySettings("       -w  eci_get_specific_reply_wait       Default: "+testApp.bWait);
               testApp.displaySettings("       -2  2 sec delay between async calls   Default: "+testApp.bAsyncDelay);
               testApp.displaySettings("       -c  clear outstanding replies first   Default: "+testApp.bClearReplies);
               testApp.displaySettings("       -i  # iterations                      Default: "+testApp.sIterations);
               testApp.displaySettings("       -d  delay between iterations          Default: "+testApp.sDelay+" seconds");
               testApp.displaySettings("       -u  userid                            Default: "+testApp.sUserid);
               testApp.displaySettings("       -p  password                          Default: "+testApp.sPassword);
               testApp.displaySettings("       -!  debug                             Default: "+testApp.bDebug);
               testApp.displaySettings("       -?  This help panel");
               testApp.displaySettings("\nNote: The -2 option causes a 2 second delay between each asynchronous call.");
               testApp.displaySettings("      The -c option clears outstanding replies.");
               return;
            }
         // end FOR loop
         }
      // end-if-args
      }
   // now go and do the work
   testApp.run();
   // end-main
   }

   // ------------------------------------------------------------------------
   // Method: run
   // Real work starts here
   // ------------------------------------------------------------------------
   public void run() {

      // if JGate not specified then use local hostname.
      // may require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            displaySettings("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

      // check if server specified. If not, server ECI can be used if on
      // same machine as gateway, and server ECI module is earlier in LIBPATH
      // than a CICS Client one. Else use first server in Client CICSCLI.INI.
      if (sServer.equals("")) {
         sUseServer = "(not specified)";
      }
      else {
         sUseServer = sServer;
      }
      displaySettings("\nASMJ002 - ECI Asynchronous Calls with Message Qualifiers") ;
      displaySettings("  -j  JGate                            -> "+sJGate);
      displaySettings("  -r  JGate port                       -> "+sPort);
      displaySettings("  -s  CICS server                      -> "+sUseServer);
      displaySettings("  -a  application                      -> "+sProgname);
      displaySettings("  -w  eci_get_specific_reply_wait      -> "+bWait);
      displaySettings("  -2  2 sec delay between async calls  -> "+bAsyncDelay);
      displaySettings("  -c  clear outstanding replies first  -> "+bClearReplies);
      displaySettings("  -i  # iterations                     -> "+sIterations);
      displaySettings("  -d  delay between iterations         -> "+sDelay+" seconds");
      displaySettings("  -u  userid                           -> "+sUserid);
      displaySettings("  -p  password                         -> "+sPassword);
      displaySettings("  -!  debug                            -> "+bDebug);
      displaySettings("  -?  for help panel");

      // create connection to JGate
      JavaGateway jgaConnection;
      jgaConnection=null;
      try {
         jgaConnection = new JavaGateway(sJGate, iPort);
         displayMsg("\n--- Successfully created JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Failed to create JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
         return;
      }

      // if server name has not been provided, get the server name
      if (sServer.equals("")) {
         ECIRequest eciRequest = ECIRequest.listSystems(1);
         try {
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            displayMsg("\n--- ERROR during flow of listSystems(1) request ---");
            return;
         }
         // output info on the Java Vector
         displaySettings("\n--- Using CICS Region: "+ eciRequest.SystemList.elementAt(0));
      }

      // ------------------------------------------------------------------
      // Use Basic ECIRequest() constructor to clear outstanding replies
      // ------------------------------------------------------------------
      // If -c option specified clear outstanding replies.
      if (bClearReplies) {

         // output what is happening
         displayMsg("\nRequest to clear outstanding replies");

         boolean bMoreReplies = true;
         while (bMoreReplies) {
            ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_GET_REPLY,
                                                     sServer,
                                                     sUserid,
                                                     sPassword,
                                                     sProgname,
                                                     sTransid,
                                                     Commarea);
            // Flow get_reply request
            try {
               jgaConnection.flow(eciRequest);
            }
            catch (Exception e) {
               GatewayTrace.traceln("ERROR: Caught Exception: " + e);
               displayMsg("\n--- Error during eci_get_reply flow ---\n");
               displayMsg("--- Caught Exception: " + e);
               return;
            }
            switch(eciRequest.Cics_Rc) {
               case ECIRequest.ECI_NO_ERROR :
                  displayMsg("-> Cleared Message_Qualifier "+eciRequest.Message_Qualifier);
                  break;
               case ECIRequest.ECI_ERR_NO_REPLY :
                  displayMsg("No more outstanding replies");
                  bMoreReplies = false;
                  break;
               case ECIRequest.ECI_ERR_TRANSACTION_ABEND :
                  if (eciRequest.Abend_Code.equals("AEY7")) {
                     displayMsg("Ignoring security error");
                     break;
                  }
               // if not AEY7 or other codes above - finish.
               default :
                  displayMsg("Unexpected return code on eci_get_reply: "+
                              eciRequest.Cics_Rc);
                  return;
            } // end-switch
         } // end-while
      } // end-if
      // ------------------------------------------------------------------
      // Full Contructor ECIRequest() required for asynchronous calls
      // ------------------------------------------------------------------

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=iIterations; iIt_Count++) {

         // output banner every iteration
         displayMsg("\n#  Call + Message_Qualifier       Commarea-Data      Length CICS Rc  Abend");
         displayMsg("-- ------------------------------ -----------------  ------ -------  -----");

         // issue 5 async calls with message_qualifiers 5 to 1
         iMsg_Qual = 5;
         for (iEci_Count=1; iEci_Count<=5; iEci_Count++) {
            ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_ASYNC,
                                                     sServer,
                                                     sUserid,
                                                     sPassword,
                                                     sProgname,
                                                     sTransid,
                                                     Commarea,
                                                     0,
                                                     ECIRequest.ECI_NO_EXTEND,
                                                     ECIRequest.ECI_LUW_NEW,
                                                     iMsg_Qual,
                                                     null );

            // set commarea to dashes
            for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
               // reset commarea to dashes
               eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            String sDashData  = new String(eciRequest.Commarea,0);

            // Flow asynchronous request
            try {
               iCallsIssued++;
               jgaConnection.flow(eciRequest);
            }
            catch (Exception e) {
               GatewayTrace.traceln("ERROR: Caught Exception: " + e);
               displayMsg("\n--- Error during eci_async flow ---\n");
               displayMsg("--- Caught Exception: " + e);
               break iteration_loop;
            }

            // keep tally of successful calls
            if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
               iReplyOK++;

            displayMsg(iIt_Count+"  Asynchronous call + MsgQual " + iMsg_Qual + "  " +
                        sDashData.substring(0,eciRequest.Commarea_Length-1) +
                        "  " + eciRequest.Commarea_Length +"     "+
                        eciRequest.Cics_Rc + "        "+eciRequest.Abend_Code );

            // for debug purposes
            GatewayTrace.traceln("Back again : AbendCode = " +
                                  eciRequest.Abend_Code);
            GatewayTrace.traceln("eci RC " + eciRequest.Cics_Rc);
            // if async call fails do not carry on ..
            if (eciRequest.Cics_Rc != 0) {
               displayMsg("\nUnexpected return code: "+eciRequest.Cics_Rc);
               displayMsg("\nSince asynchronous call failed, program is terminating.");
               return;
            } // end-async-error
            // If -2 option specified delay next async call for 2 seconds.
            // This allows the user to see the different timestamps
            // on the get_reply responses, and ensure they are in reverse
            // order.
            if (bAsyncDelay)
               try {
                  Thread.sleep(2000);
               }
               catch (Exception e) {
                  GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                  displayMsg("\n--- Error during sleep() ---\n");
                  displayMsg("--- Caught Exception: " + e);
                  break iteration_loop;
               }
            // set Message_Qualifier to one less than previous
            iMsg_Qual--;
         } // end-iEci_Count loop to issue async calls

         // ------------------------------------------------------------------
         // ECI_GET_REPLY_WAIT or ECI_GET_REPLY (loop up to 9 seconds)
         // ------------------------------------------------------------------

         // ------------------------------------------------------------------
         // Use Basic Constructor ECIRequest() for REPLY
         // ------------------------------------------------------------------
         ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_GET_SPECIFIC_REPLY_WAIT,
                                                  sServer,
                                                  sUserid,
                                                  sPassword,
                                                  sProgname,
                                                  sTransid,
                                                  Commarea);

         // if -w option specified as input parameter issue eci_get_reply_wait
         if (bWait) {
            eciRequest.Call_Type = ECIRequest.ECI_GET_SPECIFIC_REPLY_WAIT;

            iMsg_Qual = 1;
            for (iEci_Count=1; iEci_Count<=5; iEci_Count++) {
               eciRequest.Message_Qualifier = iMsg_Qual;
               sGetReply = "eci_get_specific_reply_wait "+iMsg_Qual;
               // flow ECI request
               try {
                  iCallsIssued++;
                  jgaConnection.flow(eciRequest);
               }
               catch (Exception e) {
                  GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                  displayMsg("\n--- Error during eci_specific_reply_wait flow ---\n");
                  displayMsg("--- Caught Exception: " + e);
                  break iteration_loop;
               }
               // // this next bit could be in own class
               // Convert ASCII bytes to ASCII characters
               String ComData  = new String(eciRequest.Commarea,0);

               // Display result
               displayMsg(iIt_Count+"  "+sGetReply+"  "+
                           ComData.substring(0,eciRequest.Commarea_Length-1) +
                           "  " + eciRequest.Commarea_Length +"     "+
                           eciRequest.Cics_Rc + "        " +
                           eciRequest.Abend_Code );
               // increment Message_Qualifier for next eci call
               iMsg_Qual++;
               // keep tally of successful return of commarea
               if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
                  iReplyOK++;
               } // end-if
            } // end-iEci_Count
         }  // end-if-bWait
         else {
            // -w option not specified so issue enter
            //           eci_get_specific_reply loop
            iMsg_Qual = 1;
            for (iEci_Count=1; iEci_Count<=5; iEci_Count++) {
               bReply_Rcvd = false;
               eciRequest.Message_Qualifier = iMsg_Qual;
               eciRequest.Call_Type = ECIRequest.ECI_GET_SPECIFIC_REPLY;
               iCallsIssued++; // only want to increment this total once
               for (iReply_Count=1; (iReply_Count<=9 && !bReply_Rcvd);
                    iReply_Count++) {
                  // flow ECI request
                  try {
                     jgaConnection.flow(eciRequest);
                  }
                  catch (Exception e) {
                     GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                     displayMsg("\n--- Error during eci_specific_reply flow ---\n");
                     displayMsg("--- Caught Exception: " + e);
                     break iteration_loop;
                  }
                  sGetReply = "eci_get_specific_reply#"+iReply_Count+" MQ "+iMsg_Qual;
                  // check return code ...
                  switch(eciRequest.Cics_Rc) {
                     case ECIRequest.ECI_NO_ERROR :
                        bReply_Rcvd = true;
                        break;
                     case ECIRequest.ECI_ERR_NO_REPLY :
                        // Delay next ECI call for one second (up to 9 secs)
                        try {
                           Thread.sleep(1000);
                        }
                        catch (Exception e) {
                           GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                           displayMsg("\n--- Error during sleep() ---\n");
                           displayMsg("--- Caught Exception: " + e);
                           break iteration_loop;
                        }
                        break;
                     default :
                        displayMsg(iIt_Count+"  "+sGetReply+"  "+
                                    "Unexpected return code:   " +
                                    eciRequest.Cics_Rc + "        " +
                                    eciRequest.Abend_Code );
                        return;
                  } // end-switch
               } // end-for-iReply_Count loop
               // Convert ASCII bytes to ASCII characters
               String ComData  = new String(eciRequest.Commarea,0);
               // Display result
               displayMsg(iIt_Count+"  "+sGetReply+"  "+
                           ComData.substring(0,eciRequest.Commarea_Length-1) +
                           "  " + eciRequest.Commarea_Length +"     "+
                           eciRequest.Cics_Rc + "        " +
                           eciRequest.Abend_Code );
               // increment Message_Qualifier for next eci call
               iMsg_Qual++;
               // keep tally of successful return of commarea
               if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
                  iReplyOK++;
               } // end-if
            } // end-for-iEci_Count loop
         } // end-else

         // Delay next ECI call if delay parameter specified
         try {
            Thread.sleep(iDelay);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during sleep() ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }
      } //end iIt_Count
      // ------------------------------------------------------------------
      // Close connection to JGate
      // ------------------------------------------------------------------
      try {
         jgaConnection.close();
         displayMsg("\n--- Successfully closed JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Error closing JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
      }

      displaySettings("\n--- Calls successful ... "+
                 +iReplyOK + " out of " + iCallsIssued );
      displaySettings("--- Iterations run ..... "+ (iIt_Count-1)
                           + " out of " + iIterations );

   } // end-run() method

} // end-asmj002 class
