//Ŀ
//                                                                       
//  MODULE NAME      CA83.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME ECI Samples for the CICS Gateway for Java           
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA83 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
// asyj002 - ECI_NO_EXTEND asynchronous calls with either
//           ECI_GET_REPLY_WAIT or ECI_GET_REPLY loop
//
// Written : Apr 97 - Alan Hollingshead
// Updated : Jul 97 - default application now SRVTIME, not FAADECIC
//
// asyj002 is a Java program that on each requested iteration
// issues an asynchronous call (without callback or message qualifier).
// This is followed either by an ECI_GET_REPLY_WAIT (if the -w option
// is specified as an override), or an ECI_GET_REPLY (the default).
// In the latter case up to nine ECI_GET_REPLY calls are issued to
// obtain the response from the server. Between each attempt to
// receive this reply there is a one second delay to allow for network
// traffic or a busy server.
//
// With ECI_GET_REPLY calls the output shows which call obtained the
// response. For example, "eci_get_reply #3" means that the response was
// obtained on the third attempt. If the result shows "eci_get_reply #9"
// with a CICS Rc of -5, this indicates that the response was not
// received after nine attempts. After all iterations have run, a banner
// displays the number of successful iterations where asynchronous calls
// returned a positive reply.
//
// Prior to each asynchronous call the commarea is initialised to
// 18 dashes. The server returns the current date and timestamp.
// The default CICS server program is the supplied SRVTIME
// which is compatible on all CICS servers. If desired, a different server
// program can be supplied as an input parameter to this applet/application.
// This program contains no data conversion for the commarea.
//
// If run as an applet - parameters are passed via asyj002.html.
// If run as an application - parameters can be overridden via command line.
// Issue "asyj002 -?" for help and the default settings.
//
//
import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.applet.*;

public class asyj002 extends Applet implements Runnable {
   // set up variables
   int iEci_call, iCounter, iIt_Count, iReply_Count;
   int     iIterations = 1;
   int     iPort       = 2006;
   int     iDelay      = 0;
   int     iReplyOK    = 0;
   int     iCallsIssued = 0;
   String  sDebug      = " ";
   String  sWait       = " ";
   String  sGetReply   = "";
   boolean bReply_Rcvd = false;

   // CICS commarea
   byte[] Commarea = new byte[18];

   // application defaults
   String sServer      = "" ;
   String sUseServer   = "" ;
   String sUserid      = "SYSAD" ;
   String sPassword    = "SYSAD" ;
   String sProgname    = "SRVTIME" ;
   String sTransid     = "CPMI" ;
   boolean bDebug      = false ;
   boolean bWait       = false ;
   String sJGate       = "" ;
   String sPort        = "2006" ;
   String sIterations  = "1";
   String sDelay       = "0";

   // Text area for program output
   TextArea texMessages;
   TextArea texSettings;

   // ------------------------------------------------------------------
   // Methods: displaySettings and displayMsg
   // When running as an applet ........ output sent to TextArea
   // When running as an application ... output sent to console
   // ------------------------------------------------------------------
   public void displaySettings(String strMessage) {
      if (texSettings != null) {
         texSettings.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }
   public void displayMsg(String strMessage) {
      if (texMessages != null) {
         texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }

   // ------------------------------------------------------------------
   // Method: init
   // Reads relevant parameters from html file when running as an applet
   // ------------------------------------------------------------------
   public void init() {
      // read applet parameters from html file
      sJGate      = getParameter("j");
      sPort       = getParameter("r");
      sServer     = getParameter("s");
      sProgname   = getParameter("a");
      // sNum_LUWs   = getParameter("l");
      // iNum_LUWs   = Integer.parseInt(sNum_LUWs);
      sIterations = getParameter("i");
      iIterations = Integer.parseInt(sIterations);
      sUserid     = getParameter("u");
      sPassword   = getParameter("p");
      sDelay      = getParameter("d");
      // convert to integer, and multiply to make seconds
      iDelay      = Integer.parseInt(sDelay) * 1000;
      sWait       = getParameter("w");
      sDebug      = getParameter("!");
      // convert to boolean
      if (sDebug.equals("true")) bDebug = true;
      if (sWait.equals("true"))  bWait  = true;

      // Create a text area to display messages
      setLayout(null);
      Color lightYellow = new Color(240,240,204);
      setBackground(lightYellow);

      // add textareas
      texSettings = new TextArea();
      texMessages = new TextArea();
      add(texMessages);
      add(texSettings);

      // distance from left, top, then width, height
      texSettings.reshape(10, 130, 380, 290);
      texMessages.reshape(390, 30, 500, 400);

      // define font and colours
      texSettings.setFont(new Font("Courier", Font.PLAIN, 11));
      texSettings.setEditable(false);
      texSettings.setForeground(Color.darkGray);
      texSettings.setBackground(Color.white);

      texMessages.setFont(new Font("Courier", Font.PLAIN, 11));
      texMessages.setEditable(false);
      texMessages.setForeground(Color.blue);
      texMessages.setBackground(Color.white);

      Thread test = new Thread(this);
      this.show();
      this.run();
   }

   // ------------------------------------------------------------------
   // Method: paint
   // Display headings above the textarea
   // ------------------------------------------------------------------
   public void paint(Graphics g) {
      // simple heading
      g.setFont(new Font("Helvetica", Font.BOLD, 14));
      Color purple = new Color(39,25,204);
      g.setColor(purple);
      // distance from left, top
      g.drawString("ASYJ002", 10, 50);
      g.drawString("ECI Asynchronous Calls", 10, 80);
      g.drawString("(Without Callback or Message Qualifiers)", 10, 110);
   }

   // ------------------------------------------------------------------
   // Method: main
   // read in parameters when run as an application
   // ------------------------------------------------------------------
   static public void main(String[] args) {
      // declare and instantiate testApp object based on class asyj002
      asyj002 testApp = new asyj002();

      // parse arguments entered on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            if (args[loop].indexOf("-!") > -1) {
               // set debug ON
               testApp.bDebug=true ;
               // for debug purposes
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            if (args[loop].indexOf("-w") > -1) {
               // use eci_get_reply_wait
               testApp.bWait=true ;
            }
            else if (args[loop].indexOf("-s") > -1) {
               // set cics server
               testApp.sServer = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-j") > -1) {
               // set jgate server
               testApp.sJGate = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-r") > -1) {
               // set jgate port
               testApp.sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iPort = Integer.parseInt(testApp.sPort);
            }
            else if (args[loop].indexOf("-u") > -1) {
               // set userid
               testApp.sUserid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-p") > -1) {
               // set password
               testApp.sPassword = args[loop+1] ;
                ++loop ;
            }
            else if (args[loop].indexOf("-a") > -1) {
               // set progname
               testApp.sProgname = args[loop+1] ;
               ++loop ;
            }
            // else if (args[loop].indexOf("-l") > -1) {
            //    // set Num_LUWs
            //    testApp.sNum_LUWs = args[loop+1] ;
            //    ++loop ;
            //    // convert to integer
            //    testApp.iNum_LUWs = Integer.parseInt(testApp.sNum_LUWs);
            // }
            else if (args[loop].indexOf("-i") > -1) {
               // set Iterations
               testApp.sIterations = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iIterations = Integer.parseInt(testApp.sIterations);
            }
            else if (args[loop].indexOf("-d") > -1) {
               // set Delay between each ECI call
               testApp.sDelay = args[loop+1] ;
               ++loop ;
               // convert to integer, and multiply to make seconds
               testApp.iDelay = Integer.parseInt(testApp.sDelay) * 1000;
            }
            else if ((args[loop].indexOf("-?") > -1) ||
                     (args[loop].indexOf("help") > -1)) {
               // HELP
               testApp.displaySettings("\nASYJ002 - ECI Asynchronous Call Testing") ;
               testApp.displaySettings("Usage: java asyj002 <parameters>");
               testApp.displaySettings("       -j  JGate               Default: (local hostname)");
               testApp.displaySettings("       -r  JGate port          Default: "+testApp.sPort);
               testApp.displaySettings("       -s  CICS server         Default: (not specified)");
               testApp.displaySettings("       -a  application         Default: "+testApp.sProgname);
               testApp.displaySettings("       -w  eci_get_reply_wait  Default: "+testApp.bWait);
               testApp.displaySettings("       -i  # iterations        Default: "+testApp.sIterations);
               testApp.displaySettings("       -d  delay               Default: "+testApp.sDelay+" seconds");
               testApp.displaySettings("       -u  userid              Default: "+testApp.sUserid);
               testApp.displaySettings("       -p  password            Default: "+testApp.sPassword);
               testApp.displaySettings("       -!  debug               Default: "+testApp.bDebug);
               testApp.displaySettings("       -?  This help panel");
               return;
            }
         // end FOR loop
         }
      // end-if-args
      }
   // now go and do the work
   testApp.run();
   // end-main
   }

   // ------------------------------------------------------------------------
   // Method: run
   // Real work starts here
   // ------------------------------------------------------------------------
   public void run() {

      // if JGate not specified then use local hostname.
      // may require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            displaySettings("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

      // check if server specified. If not, server ECI can be used if on
      // same machine as gateway, and server ECI module is earlier in LIBPATH
      // than a CICS Client one. Else use first server in Client CICSCLI.INI.
      if (sServer.equals("")) {
         sUseServer = "(not specified)";
      }
      else {
         sUseServer = sServer;
      }
      displaySettings("\nASYJ002 - ECI Asynchronous Call Testing") ;
      displaySettings("       -j  JGate              -> "+sJGate);
      displaySettings("       -r  JGate port         -> "+sPort);
      displaySettings("       -s  CICS server        -> "+sUseServer);
      displaySettings("       -a  application        -> "+sProgname);
      displaySettings("       -w  eci_get_reply_wait -> "+bWait);
      displaySettings("       -i  # iterations       -> "+sIterations);
      displaySettings("       -d  delay              -> "+sDelay+" seconds");
      displaySettings("       -u  userid             -> "+sUserid);
      displaySettings("       -p  password           -> "+sPassword);
      displaySettings("       -!  debug              -> "+bDebug);
      displaySettings("       -?  for help panel");

      // create connection to JGate
      JavaGateway jgaConnection;
      jgaConnection=null;
      try {
         jgaConnection = new JavaGateway(sJGate, iPort);
         displayMsg("\n--- Successfully created JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Failed to create JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
         return;
      }

      // if server name has not been provided, get the server name
      if (sServer.equals("")) {
         ECIRequest eciRequest = ECIRequest.listSystems(1);
         try {
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            displayMsg("\n--- ERROR during flow of listSystems(1) request ---");
            return;
         }
         // output info on the Java Vector
         displaySettings("\n--- Using CICS Region: "+ eciRequest.SystemList.elementAt(0));
      }

      // ------------------------------------------------------------------
      // Full Contructor ECIRequest() required for asynchronous calls
      // ------------------------------------------------------------------

      displayMsg("\n#  Call               Commarea-Data      Length CICS Rc  Abend");
      displayMsg("-- ------------------ -----------------  ------ -------  -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=iIterations; iIt_Count++) {

         // output banner every 10 iterations
         if (iIt_Count%10==0) {
            displayMsg("\n#   Call               Commarea-Data      Length CICS Rc  Abend");
            displayMsg("--  ------------------ -----------------  ------ -------  -----");
         } // end-if

         ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_ASYNC,
                                                  sServer,
                                                  sUserid,
                                                  sPassword,
                                                  sProgname,
                                                  sTransid,
                                                  Commarea,
                                                  0,
                                                  ECIRequest.ECI_NO_EXTEND,
                                                  ECIRequest.ECI_LUW_NEW,
                                                  0,
                                                  null );

         // set commarea to dashes
         for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
            // reset commarea to dashes
            eciRequest.Commarea[iCounter]=45;
            // Convert ASCII bytes to ASCII characters
            String sDashData  = new String(eciRequest.Commarea,0);

         // Flow asynchronous request
         try {
            iCallsIssued++;
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during eci_async flow ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }

         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR)
            iReplyOK++;

         displayMsg(iIt_Count+"  Async call ->      " +
                     sDashData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc +"        "+ eciRequest.Abend_Code );

         // for debug purposes
         GatewayTrace.traceln("Back again : AbendCode = " +
                               eciRequest.Abend_Code);
         GatewayTrace.traceln("eci RC " + eciRequest.Cics_Rc);

         // ------------------------------------------------------------------
         // ECI_GET_REPLY_WAIT or ECI_GET_REPLY (loop up to 9 seconds)
         // ------------------------------------------------------------------

         // if -w option specified as input parameter issue eci_get_reply_wait

         iCallsIssued++;

         if (bWait) {
            eciRequest.Call_Type = ECIRequest.ECI_GET_REPLY_WAIT;

            sGetReply = "eci_get_reply_wait ";
            // flow ECI request
            try {
               jgaConnection.flow(eciRequest);
            }
            catch (Exception e) {
               GatewayTrace.traceln("ERROR: Caught Exception: " + e);
               displayMsg("\n--- Error during eci_get_reply_wait flow ---\n");
               displayMsg("--- Caught Exception: " + e);
               break iteration_loop;
            }
         }
         else {
            // -w option not specified so issue enter eci_get_reply loop
            bReply_Rcvd = false;
            for (iReply_Count=1; (iReply_Count<=9 && !bReply_Rcvd);
                 iReply_Count++) {
               eciRequest.Call_Type = ECIRequest.ECI_GET_REPLY;

               // flow ECI request
               try {
                  jgaConnection.flow(eciRequest);
               }
               catch (Exception e) {
                  GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                  displayMsg("\n--- Error during eci_get_reply flow ---\n");
                  displayMsg("--- Caught Exception: " + e);
                  break iteration_loop;
               }
               sGetReply = "eci_get_reply #"+iReply_Count+"   ";
               // check for good return code ...
               if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
                  bReply_Rcvd = true;
               }
               // else delay for one second and try again (up to 9 times)
               else {
                  // Delay next ECI call for one second
                  try {
                     Thread.sleep(1000);
                  }
                  catch (Exception e) {
                     GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                     displayMsg("\n--- Error during sleep() ---\n");
                     displayMsg("--- Caught Exception: " + e);
                     break iteration_loop;
                  }
               } // end-else
            } // end-for loop
         } // end-else

         // Convert ASCII bytes to ASCII characters
         String ComData  = new String(eciRequest.Commarea,0);

         // Display result
         displayMsg(iIt_Count+"  "+sGetReply+
                     ComData.substring(0,eciRequest.Commarea_Length-1) +
                     "  " + eciRequest.Commarea_Length +"     "+
                     eciRequest.Cics_Rc +"        "+ eciRequest.Abend_Code );

         // keep tally of successful return of commarea
         if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
            iReplyOK++;
         } // end-if
         // Delay next ECI call if delay parameter specified
         try {
            Thread.sleep(iDelay);
         }
         catch (Exception e) {
            GatewayTrace.traceln("ERROR: Caught Exception: " + e);
            displayMsg("\n--- Error during sleep() ---\n");
            displayMsg("--- Caught Exception: " + e);
            break iteration_loop;
         }
      } //end iIt_Count
      // ------------------------------------------------------------------
      // Close connection to JGate
      // ------------------------------------------------------------------
      try {
         jgaConnection.close();
         displayMsg("\n--- Successfully closed JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Error closing JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
      }

      displaySettings("\n--- Calls successful ... "+
                 +iReplyOK + " out of " + iCallsIssued );
      displaySettings("--- Iterations run ..... "+ (iIt_Count-1)
                           + " out of " + iIterations );

   } // end-run() method

} // end-asyj002 class
