//Ŀ
//                                                                       
//  MODULE NAME      CA83.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME ECI Samples for the CICS Gateway for Java           
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA83 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
// ecijlst - List all systems available; then for each server issue:
//           (1) eci state async call
//           (2) eci sync call with commarea to server application
//           (3) eci state sync call
//
//           Program completes with a status summary for all servers.
//           The status summary shown is for the eci state sync call.
//
// Written : May 97 - Alan Hollingshead
// Updated : Jul 97 - default application now SRVTIME, not FAADECIC
//
// This source file creates two classes:
// (1) ecijlst.class .......... issues the state and eci_sync calls.
// (2) TestCBst.class ......... handles all callbacks received.
//
// Issuing "java ecijlst -?" shows the following help and defaults:
//
//      ECIJLST - List Systems and Check Status
//      Usage: java ecijlst <parameters>
//             -j  JGate              Default: (local hostname)
//             -r  JGate port         Default: 2006
//             -a  application        Default: SRVTIME
//             -u  userid             Default: SYSAD
//             -p  password           Default: SYSAD
//             -!  debug              Default: false
//             -?  This help panel
//
// Example: java ecijlst -j myJGate
//      The above example overrides the -j parameter with "myJGate".
//
// Parameter overrides:
//      -j ... hostname or IP address where Java Gateway is running.
//             The default is your local hostname.
//      -p ... Java Gateway port. This defaults to 2006 unless
//             otherwise specified when starting the Java Gateway.
//      -a ... name of server application.
//      -u ... userid, if required by the server mirror transaction.
//      -p ... password, if required by the server mirror transaction.
//      -! ... some debug info is supplied if "-!" is specified.
//      -? ... displays the help panel and defaults.
//
// If run as an applet - parameters are passed via ecijlst.html.
// If run as an application - parameters can be overridden via command line.
//
// This applet/application will work for up to 100 systems.
//
// RECOMMENDATION:
//      If using a (common) CICS Client, issue "CICSCLI /N" before
//      running this program to suppress unwanted client pop-ups.
//      Any CICS Client errors will continue to be written to the
//      "CICSCLI.LOG" error file.
//
import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.applet.*;

public class ecijlst extends Applet implements Runnable {
   // set up variables
   int iEci_call, iCounter, iReply_Count;
   int     iPort       = 2006;
   int     iReplyOK    = 0;
   String  sDebug      = " ";
   boolean bReply_Rcvd = false;
   JavaGateway jgaConnection = null;

    static int iCBtoRun         = 0;
    static int iCBrun           = 0;
    Object ObjLock       = null;

   // CICS commarea
   byte[] Commarea = new byte[18];

   // application defaults
   String sServer      = "" ;
   static String sUserid      = "SYSAD" ;
   static String sPassword    = "SYSAD" ;
   static String sProgname    = "SRVTIME" ;
   static String sTransid     = "CPMI" ;
   boolean bDebug      = false ;
   boolean bWait       = false ;
   static String sJGate       = "" ;
   static String sPort        = "2006" ;
   int iTotalSystems = 0;
   // remember arrays use indice zero
   String sArraySummary[][] = new String[99][6];
   String sIterations  = "1";
   String sDelay       = "0";

   // Text area for program output
   TextArea texMessages;
   TextArea texSettings;

   // ------------------------------------------------------------------
   // Methods: displaySettings and displayMsg
   // When running as an applet ........ output sent to TextArea
   // When running as an application ... output sent to console
   // ------------------------------------------------------------------
   public void displaySettings(String strMessage) {
      if (texSettings != null) {
         texSettings.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }
   public void displayMsg(String strMessage) {
      if (texMessages != null) {
         texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }

   // ------------------------------------------------------------------
   // Method: init
   // Reads relevant parameters from html file when running as an applet
   // ------------------------------------------------------------------
   public void init() {
      // read applet parameters from html file
      sJGate      = getParameter("j");
      sPort       = getParameter("r");
      sProgname   = getParameter("a");
      sUserid     = getParameter("u");
      sPassword   = getParameter("p");
      sDebug      = getParameter("!");
      // convert to boolean
      if (sDebug.equals("true")) bDebug = true;

      // Create a text area to display messages
      setLayout(null);
      Color lightYellow = new Color(240,240,204);
      setBackground(lightYellow);

      // add textareas
      texSettings = new TextArea();
      texMessages = new TextArea();
      add(texMessages);
      add(texSettings);

      // distance from left, top, then width, height
      texSettings.reshape(5, 80, 500, 350);
      texMessages.reshape(505, 30, 480, 400);

      // define font and colours
      texSettings.setFont(new Font("Courier", Font.PLAIN, 11));
      texSettings.setEditable(false);
      texSettings.setForeground(Color.darkGray);
      texSettings.setBackground(Color.white);

      texMessages.setFont(new Font("Courier", Font.PLAIN, 11));
      texMessages.setEditable(false);
      texMessages.setForeground(Color.blue);
      texMessages.setBackground(Color.white);

      Thread test = new Thread(this);
      this.show();
      this.run();
   }

   // ------------------------------------------------------------------
   // Method: paint
   // Display headings above the textarea
   // ------------------------------------------------------------------
   public void paint(Graphics g) {
      // simple heading
      g.setFont(new Font("Helvetica", Font.BOLD, 14));
      Color purple = new Color(39,25,204);
      g.setColor(purple);
      // distance from left, top
      g.drawString("ECIJLST", 10, 30);
      g.drawString("List Systems and Check Status", 10, 60);
   }

   // ------------------------------------------------------------------
   // Method: main
   // read in parameters when run as an application
   // ------------------------------------------------------------------
   static public void main(String[] args) {
      // declare and instantiate testApp object based on class ecijlst
      ecijlst testApp = new ecijlst();

      // parse arguments entered on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            if (args[loop].indexOf("-!") > -1) {
               // set debug ON
               testApp.bDebug=true ;
               // for debug purposes
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            else if (args[loop].indexOf("-j") > -1) {
               // set jgate server
               testApp.sJGate = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-r") > -1) {
               // set jgate port
               testApp.sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iPort = Integer.parseInt(testApp.sPort);
            }
            else if (args[loop].indexOf("-u") > -1) {
               // set userid
               testApp.sUserid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-p") > -1) {
               // set password
               testApp.sPassword = args[loop+1] ;
                ++loop ;
            }
            else if (args[loop].indexOf("-a") > -1) {
               // set progname
               testApp.sProgname = args[loop+1] ;
               ++loop ;
            }
            else if ((args[loop].indexOf("-?") > -1) ||
                     (args[loop].indexOf("help") > -1)) {
               // HELP
               testApp.displaySettings("\nECIJLST - List Systems and Check Status") ;
               testApp.displaySettings("Usage: java ecijlst <parameters>");
               testApp.displaySettings("       -j  JGate              Default: (local hostname)");
               testApp.displaySettings("       -r  JGate port         Default: "+testApp.sPort);
               testApp.displaySettings("       -a  application        Default: "+testApp.sProgname);
               testApp.displaySettings("       -u  userid             Default: "+testApp.sUserid);
               testApp.displaySettings("       -p  password           Default: "+testApp.sPassword);
               testApp.displaySettings("       -!  debug              Default: "+testApp.bDebug);
               testApp.displaySettings("       -?  This help panel");
               return;
            }
         // end FOR loop
         }
      // end-if-args
      }
   // now go and do the work
   testApp.run();
   // end-main
   }

   // ------------------------------------------------------------------------
   // Method: run
   // Real work starts here
   // ------------------------------------------------------------------------
   public void run() {

      // if JGate not specified then use local hostname.
      // may require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            displaySettings("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

      displaySettings("\nECIJLST - List Systems and Check Status") ;
      displaySettings("       -j  JGate              -> "+sJGate);
      displaySettings("       -r  JGate port         -> "+sPort);
      displaySettings("       -a  application        -> "+sProgname);
      displaySettings("       -u  userid             -> "+sUserid);
      displaySettings("       -p  password           -> "+sPassword);
      displaySettings("       -!  debug              -> "+bDebug);
      displaySettings("       -?  Help panel");

      // create connection to JGate
      try {
         jgaConnection = new JavaGateway(sJGate, iPort);
         displayMsg("\n--- Successfully created JavaGateway connection ---\n");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Failed to create JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
         return;
      }

      // list available systems

      ECIRequest eciRequest = null;
      eciRequest = ECIRequest.listSystems(99);
      try {
         jgaConnection.flow(eciRequest);

         iTotalSystems = (eciRequest.SystemList.size()/2);

         displayMsg("Servers available: ");

         if (eciRequest.getRc() == 0) {
             for (int iSysLoop = 0, iSysNum = 1;
                      iSysLoop < eciRequest.SystemList.size();
                      iSysLoop += 2,
                      iSysNum  += 1) {

                displayMsg("     Server ("+iSysNum+") " +
                      (String) eciRequest.SystemList.elementAt(iSysLoop) +
                      ", Description: " +
                      (String) eciRequest.SystemList.elementAt(iSysLoop + 1));
             }
         }
         else
         {
            displayMsg("ECIRequest.listSystems failed with Rc : " +
                        eciRequest.getRc());
            return;
         }
         // --------------------------------------------------------
         // For each server issue: eci state async call
         //                        (as part of getStatus method)
         //                        eci sync call with commarea to SRVTIME
         //                        eci state sync call
         //                        (as part of getStatus method)
         // --------------------------------------------------------

         String sThisServer = null;

         for (int iSysLoop = 0, iSysNum = 1;
                  iSysLoop < eciRequest.SystemList.size();
                  iSysLoop += 2,
                  iSysNum  += 1) {

            sThisServer = (String) eciRequest.SystemList.elementAt(iSysLoop);

            displayMsg("\n----------------------------------------------");
            displayMsg("Server ("+iSysNum+" of "+(eciRequest.SystemList.size()/2)+
                       ") "+sThisServer);
            displayMsg("----------------------------------------------");

            // remember cell zero
            sArraySummary[iSysNum-1][0] = sThisServer;

            // issue ECI_STATE_ASYNC call
            eciStateAsync(sThisServer);

            // issue ECI_SYNC call
            eciSyncCall(sThisServer, iSysNum);

            // issue ECI_STATE_SYNC call
            eciStateSync(sThisServer, iSysNum);

         } // end-for
      } // try
   catch (Exception e) {
      GatewayTrace.traceln("ERROR: Caught Exception: " + e);
      displayMsg("\n--- Error returned by JGate ---\n");
      displayMsg("--- Caught Exception: " + e);
      return;
   }
      // ------------------------------------------------------------------
      // Close connection to JGate
      // ------------------------------------------------------------------
      try {
         jgaConnection.close();
         displayMsg("\n--- Successfully closed JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Error closing JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
      }

      // all calls made - so display a summary
      displaySettings("\nStatus Summary:\n");
      displaySettings("           Connection   Server     Client         "+
                 "ECI_SYNC            CICS");
      displaySettings("Server     Status       Status     Status         "+
                 "Results             RC  ");
      displaySettings("--------   ----------   --------   ------------   "+
                 "-----------------   ----");


      for (int iSysLoop = 0; iSysLoop < (iTotalSystems); iSysLoop++) {
         displaySettings(sArraySummary[iSysLoop][0]+"   "+
                    sArraySummary[iSysLoop][1]+"   "+
                    sArraySummary[iSysLoop][2]+"   "+
                    sArraySummary[iSysLoop][3]+"   "+
                    sArraySummary[iSysLoop][4]+"  "+
                    sArraySummary[iSysLoop][5]);
      } // end-for
   } // end-run() method

   //------------------------------------------------------------------------
   // Method : eciSyncCall
   // Issue an ECI_SYNC call for this server
   //------------------------------------------------------------------------

   private void eciSyncCall(String sThisServer, int iSysNum)
                            throws IOException {
      // Issue eci_sync call
      ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                       sThisServer,
                                       sUserid,
                                       sPassword,
                                       sProgname,
                                       sTransid,
                                       Commarea,
                                       0,
                                       ECIRequest.ECI_NO_EXTEND,
                                       ECIRequest.ECI_LUW_NEW );

      // set commarea to dashes
      for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
         // reset commarea to dashes
         eciRequest.Commarea[iCounter]=45;

      // Convert ASCII bytes to ASCII characters
      String sDashData  = new String(eciRequest.Commarea,0);

      displayMsg("---> Flow eci_sync call to server: "+sThisServer);
      // Flow synchronous request
      jgaConnection.flow(eciRequest);

      // Convert ASCII bytes to ASCII characters
      String ComData  = new String(eciRequest.Commarea,0);

      displayMsg("<--- Results of eci_sync call ");
      displayMsg("     Commarea ..... "+ ComData.substring(0,eciRequest.Commarea_Length-1));
         //      +"  Length ... " + eciRequest.Commarea_Length);
      displayMsg("     CICS Rc ...... "+ eciRequest.Cics_Rc);
      displayMsg("     Abend code ... "+eciRequest.Abend_Code );

      ecijlst.sArraySummary[iSysNum-1][4] = ComData;

      ecijlst.sArraySummary[iSysNum-1][5] = String.valueOf(eciRequest.Cics_Rc);

   }
   //------------------------------------------------------------------------
   // Method : eciStateSync
   // Issue an ECI_STATE_SYNC call for this server
   // The method getStatus(String sThisServer) used here sets the values
   // of the ECIRequest object for an ECI_STATE_SYNC_JAVA call on the
   // CICS Gateway for Java. The extend_mode is set to ECI_STATE_IMMEDIATE
   // i.e. a reply is sent immediately is is available.
   //------------------------------------------------------------------------

   private void eciStateSync(String sThisServer, int iSysNum)
                             throws IOException {
      // Issue getStatus call

      ECIRequest eciRequest = (new ECIRequest()).getStatus(sThisServer);

      displayMsg("\n---> Synchronous getStatus call for server: "
                       +sThisServer);

      jgaConnection.flow(eciRequest);

      displayMsg("<--- Results of synchronous getStatus request");
      displayMsg("     Server: " + sThisServer);
      displayMsg("       Connection status ... " +
                 eciRequest.
                     stringConnectionType(eciRequest.ConnectionType));
      displayMsg("       Server status ....... " +
                 eciRequest.stringServerStatus(eciRequest.CicsServerStatus));
      displayMsg("       Client status ....... " +
                 eciRequest.stringClientStatus(eciRequest.CicsClientStatus) +
                 "\n");

      switch(eciRequest.ConnectionType) {
         case eciRequest.ECI_CONNECTED_TO_CLIENT  :
                  ecijlst.sArraySummary[iSysNum-1][1] = "To Client ";
                  break;
         case eciRequest.ECI_CONNECTED_TO_SERVER  :
                  ecijlst.sArraySummary[iSysNum-1][1] = "To Server ";
                  break;
         case eciRequest.ECI_CONNECTED_NOWHERE  :
                  ecijlst.sArraySummary[iSysNum-1][1] = "Nowhere   ";
                  break;
         default  :
                  ecijlst.sArraySummary[iSysNum-1][1] = "Unknown connection type";
                  break;
      }

      switch(eciRequest.CicsServerStatus) {
         case eciRequest.ECI_SERVERSTATE_UP      :
                  ecijlst.sArraySummary[iSysNum-1][2] = "Up      ";
                  break;
         case eciRequest.ECI_SERVERSTATE_DOWN    :
                  ecijlst.sArraySummary[iSysNum-1][2] = "Down    ";
                  break;
         case eciRequest.ECI_SERVERSTATE_UNKNOWN :
                  ecijlst.sArraySummary[iSysNum-1][2] = "Unknown ";
                  break;
         default  :
                  ecijlst.sArraySummary[iSysNum-1][2] = "Unknown CicsServerState";
                  break;
      }

      switch(eciRequest.CicsClientStatus) {
         case eciRequest.ECI_CLIENTSTATE_UP      :
                  ecijlst.sArraySummary[iSysNum-1][3] = "Up          ";
                  break;
         case eciRequest.ECI_CLIENTSTATE_INAPPLICABLE :
                  ecijlst.sArraySummary[iSysNum-1][3] = "Inapplicable";
                  break;
         case eciRequest.ECI_CLIENTSTATE_UNKNOWN :
                  ecijlst.sArraySummary[iSysNum-1][3] = "Unknown     ";
                  break;
         default  :
                  ecijlst.sArraySummary[iSysNum-1][3] = "Unknown CicsClientState";
                  break;
      }

   }
   //------------------------------------------------------------------------
   // Method : eciStateAsync
   // Issue an ECI_STATE_ASYNC call for this server
   // The method getStatus(String sThisServer, int iMessage_Qualifier, calback)
   // used here sets the values of the ECIRequest object for an
   // ECI_STATE_ASYNC_JAVA call on the CICS Gateway for Java.
   // The extend_mode is set to ECI_STATE_IMMEDIATE
   // i.e. a reply is sent immediately is is available.
   // Since this call is asynchronous, a reply is sent to the callback
   // routine when the status information is available. When this routine
   // is invoked the fields ConnectionType, CicsServerStatus and
   // CicsClientStatus will have been updated with any information the
   // server has supplied.
   //------------------------------------------------------------------------

   private void eciStateAsync(String sThisServer) throws IOException

   {
      // declare and instantiate object for lock
      ObjLock     = new Object();

      Thread thrMain        = Thread.currentThread();

      int iMessage_Qualifier = 1;
      // increment so we know how many calls have been made
      iCBtoRun++;

      int iNum_Calls = 1;
      // new calBack for each call
      TestCBst calBack  = new TestCBst(thrMain, ObjLock, texMessages);

      // Issue getStatus call
      ECIRequest eciRequest = (new ECIRequest()).getStatus(sThisServer,
                                         iMessage_Qualifier,
                                         calBack );

      displayMsg("\n---> Asynchronous getStatus call for server: "
                       +sThisServer);
      // flow ECI request
      jgaConnection.flow(eciRequest);

      // synchronize object
      synchronized(ObjLock)
      {
         while (iCBtoRun != iCBrun)
         {
            try
            {
                ObjLock.wait();
            }
            catch (Exception e)
            {
                displayMsg("Caught Exception " + e);
            }
         }
      }

      //   displayMsg("Back from wait()");

     displayMsg("<--- Results of asynchronous getStatus request");
     displayMsg("     Server: " + sThisServer);
     displayMsg("       Connection status ... " +
                eciRequest.
                    stringConnectionType(eciRequest.ConnectionType));
     displayMsg("       Server status ....... " +
                eciRequest.stringServerStatus(eciRequest.CicsServerStatus));
     displayMsg("       Client status ....... " +
                eciRequest.stringClientStatus(eciRequest.CicsClientStatus) +
                "\n");
   }
} // end-ecijlst class
// ------------------------------------------------------------------------
// Class: TestCBst (ie TestCallback for ecijlst)
// Handles all callbacks received from the Java gateway.
// ------------------------------------------------------------------------
class TestCBst implements Callbackable
{
   ECIRequest CB_eciRequest  = null;
   Thread thrToWakeUp     = null;
   Object    ObjLock      = null;
   boolean boolIsAlive    = false;
   int myCallNo           = 0;
   TextArea CB_texMessages = null;

   public TestCBst(Thread thrWakeUp, Object ObjParm, TextArea texMessages)
   {
       thrToWakeUp = thrWakeUp;
       ObjLock     = ObjParm;
       CB_texMessages = texMessages;
   }

   public void displayMsg(String strMessage) {
      if (CB_texMessages != null) {
         CB_texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }

   public void setResults(GatewayRequest gatReply)
   {
      CB_eciRequest  = (ECIRequest) gatReply;
   }

   public void run() {
      boolIsAlive = thrToWakeUp.isAlive();

      if (boolIsAlive) {
         //  displayMsg("Rousing main thread ...");

         synchronized (ObjLock) {
            ecijlst.iCBrun++;
            //     displayMsg("Notifying : "+ObjLock);
            ObjLock.notify();
         }
      }
      else {
         // big trouble as main thread has disappeared!
         displayMsg("Main thread is dead ...");
      }
   }
}   // end-TestCBst class
