//Ŀ
//                                                                       
//  MODULE NAME      CA83.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME ECI Samples for the CICS Gateway for Java           
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA83 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
// synj001 - Extended ECI Synchronous Calls with multiple LUWs
//
// Written : Nov 96 - Alan Hollingshead
// Updated : Jul 97 - default application now SRVTIME, not FAADECIC
//
// Note:
// synj001 runs only as a Java application (hence is short and simple).
// synj002 functions exactly the same as synj002 but can be run as an
//         applet or an application (hence is longer to cater for running
//         as an applet.
//
// synj001 is a Java application that on each requested iteration
// issues sets of six ECI synchronous calls for each LUW. The first
// two calls are followed by an ECI_BACKOUT. The fourth and fifth
// calls are followed by an ECI_COMMIT. The commarea on each call
// is initialised to 18 dashes. The server returns the current date
// and timestamp. The default CICS server program is the supplied SRVTIME
// which is compatible on all CICS servers. If desired, a different server
// program can be supplied as an input parameter to this applet/application.
// This program contains no data conversion for the commarea.
//
// Parameters can be overridden via command line.
// Issue "synj001 -?" for help and the default settings.
//
//
import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.io.*;
import java.net.*;

public class synj001 {
   public static void main(String[] args) {
      //  set up variables
      int iEci_call, iLUW, iCounter, iIt_Count;
      int iNum_LUWs      = 1;
      int iIterations    = 1;
      int iPort          = 2006;
      int iDelay         = 0;
      int iCallsIssued   = 0;
      int iReplyOK       = 0;

      //  CICS commarea
      byte[] Commarea = new byte[18];

      // application defaults
      String sServer     = "" ;
      String sUseServer  = "" ;
      String sUserid     = "SYSAD" ;
      String sPassword   = "SYSAD" ;
      String sProgname   = "SRVTIME" ;
      String sTransid    = "CPMI" ;
      boolean debug      = false ;
      String sJGate      = "" ;
      String sPort       = "2006" ;
      String sExt_mode   = "ECI_EXTENDED";
      String sNum_LUWs   = "1";
      String sIterations = "1";
      String sDelay      = "0";

      // Parse arguments passed in on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            if (args[loop].indexOf("-!") > -1) {
               // set debug ON
               debug=true ;
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            else if (args[loop].indexOf("-s") > -1) {
               // set cics server
               sServer = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-j") > -1) {
               // set jgate server
               sJGate = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-r") > -1) {
               // set jgate port
               sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               iPort = Integer.parseInt(sPort);
            }
            else if (args[loop].indexOf("-u") > -1) {
               // set userid
               sUserid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-p") > -1) {
               // set password
               sPassword = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-a") > -1) {
               // set progname
               sProgname = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-l") > -1) {
               // set Num_LUWs
               sNum_LUWs = args[loop+1] ;
               ++loop ;
               // convert to integer
               iNum_LUWs = Integer.parseInt(sNum_LUWs);
            }
            else if (args[loop].indexOf("-i") > -1) {
               // set Iterations
               sIterations = args[loop+1] ;
               ++loop ;
               // convert to integer
               iIterations = Integer.parseInt(sIterations);
            }
            else if (args[loop].indexOf("-d") > -1) {
               // set Delay between each ECI call
               sDelay = args[loop+1] ;
               ++loop ;
               // convert to integer, and multiply to make seconds
               iDelay = Integer.parseInt(sDelay) * 1000;
            }
            else if ((args[loop].indexOf("-?") > -1)||
                     (args[loop].indexOf("help") > -1)) {
               // HELP
               System.out.println("SYNJ001 - Extended ECI Synchronous Calls") ;
               System.out.println("USAGE: java synj001 <parameters>");
               System.out.println("       -j  JGate              Default: (local hostname)");
               System.out.println("       -r  JGate port         Default: "+sPort);
               System.out.println("       -s  CICS server        Default: (not specified)");
               System.out.println("       -a  application        Default: "+sProgname);
               System.out.println("       -l  # concurrent luws  Default: "+sNum_LUWs);
               System.out.println("       -i  # iterations       Default: "+sIterations);
               System.out.println("       -u  userid             Default: "+sUserid);
               System.out.println("       -p  password           Default: "+sPassword);
               System.out.println("       -d  delay              Default: "+sDelay+" seconds");
               System.out.println("       -!  debug              Default: "+debug);
               System.out.println("       -?  This help panel");
               return;
            }
         } // end-for loop
      } // end-if-args

      // if JGate not specified then use local hostname.
      // may require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            System.out.println("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

      // check if server specified. If not, server ECI can be used if on
      // same machine as gateway, and server ECI module is earlier in LIBPATH
      // than a CICS Client one. Else use first server in Client CICSCLI.INI.
      if (sServer.equals("")) {
         sUseServer = "(not specified)";
      }
      else {
         sUseServer = sServer;
      }
      System.out.println("\nSYNJ001 - Extended ECI Synchronous Calls") ;
      System.out.println("       -j  JGate              -> "+sJGate);
      System.out.println("       -r  JGate port         -> "+sPort);
      System.out.println("       -s  CICS server        -> "+sUseServer);
      System.out.println("       -a  application        -> "+sProgname);
      System.out.println("       -l  # concurrent luws  -> "+sNum_LUWs);
      System.out.println("       -i  # iterations       -> "+sIterations);
      System.out.println("       -u  userid             -> "+sUserid);
      System.out.println("       -p  password           -> "+sPassword);
      System.out.println("       -d  delay              -> "+sDelay+" seconds");
      System.out.println("       -!  debug              -> "+debug);
      System.out.println("       -?  for help panel");

      // ---------------------------------------------------------------------
      // Create connection to JGate
      // ---------------------------------------------------------------------
      JavaGateway jgaConnection;
      jgaConnection=null;
      try
      {
         jgaConnection = new JavaGateway(sJGate, iPort);
         System.out.println("\n--- Successfully created JavaGateway connection ---");
      }
      catch (Exception e)
      {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         System.out.println("\n--- Failed to create JavaGateway connection ---");
         System.out.println("--- Caught Exception: " + e);
         return;
      }

      // if server name has not been provided, get the server name
      if (sServer.equals("")) {
         ECIRequest eciRequest = ECIRequest.listSystems(1);
         try {
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            System.out.println("\n--- ERROR during flow of listSystems(1) request ---");
            return;
         }
         // output info on the Java Vector
         System.out.println("\n--- Using CICS Region: "+ eciRequest.SystemList.elementAt(0));
      }

      // ----------------------------------------------------------------------
      // Extended Contructor ECIRequest()
      // ----------------------------------------------------------------------
      // System.out.println("\n--- Using Extended Constructor ECIRequest() ---");

      ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                               sServer,
                                               sUserid,
                                               sPassword,
                                               sProgname,
                                               sTransid,
                                               Commarea,
                                               0,
                                               0,
                                               0);

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=iIterations; iIt_Count++) {

         // display header
         System.out.println("\n--- Iteration: "+iIt_Count+" of "+iIterations+"\n");
         System.out.println("# LUW Extend-Mode  Commarea-Data      Length CICS Rc  Abend  LUW Token");
         System.out.println("- --- -----------  -------------      ------ -------  -----  ---------");

         // array for LUW tokens
         int[] aiTokens = new int[iNum_LUWs+1];
         for (iLUW=1; iLUW<=iNum_LUWs; iLUW++)
         // initialise Tokens to zero
            aiTokens[iLUW]=0;

         for (iEci_call=1; iEci_call<=6; iEci_call++) {
            switch(iEci_call) {
               case 3  : eciRequest.Extend_Mode=ECIRequest.ECI_BACKOUT;
                         sExt_mode = "ECI_BACKOUT ";
                         break;
               case 6  : eciRequest.Extend_Mode=ECIRequest.ECI_COMMIT;
                         sExt_mode = "ECI_COMMIT  ";
                         break;
               default : eciRequest.Extend_Mode=ECIRequest.ECI_EXTENDED;
                         sExt_mode = "ECI_EXTENDED";
                         break;
            } // end-switch

            // for each LUW
            for (iLUW=1; iLUW<=iNum_LUWs; iLUW++) try {
               // set commarea to dashes
               for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++) {
                  // reset commarea to dashes
                  eciRequest.Commarea[iCounter]=45;
               }
               // use correct LUW token
               eciRequest.Luw_Token=aiTokens[iLUW];

               // flow ECI request
               iCallsIssued++;
               jgaConnection.flow(eciRequest);

               if (eciRequest.Cics_Rc == 0)
                  iReplyOK++;

               // for debug purposes
               GatewayTrace.traceln("Back again : AbendCode = " +
                                     eciRequest.Abend_Code);
               GatewayTrace.traceln("eci RC " + eciRequest.Cics_Rc);

               // Convert ASCII bytes to ASCII characters
               String ComData  = new String(eciRequest.Commarea,0);

               // Display result
               System.out.println(iEci_call+"  "+iLUW+"  "+
                    sExt_mode+" "+
                    ComData.substring(0,eciRequest.Commarea_Length-1)+"  "+
                    eciRequest.Commarea_Length +"     "+
                    eciRequest.Cics_Rc +"       "+
                    eciRequest.Abend_Code +"    "+
                    eciRequest.Luw_Token);

               // Assign new LUW token
               aiTokens[iLUW]=eciRequest.Luw_Token;

               // Delay next ECI call if delay parameter specified
               try {
                  Thread.sleep(iDelay);
               }
               catch (Exception e) {
                  GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                  System.out.println("\n--- Error during sleep() ---\n");
                  System.out.println("--- Caught Exception: " + e);
                  break iteration_loop;
               }
            }  //end LUW for-loop
            catch (Exception e) {
               GatewayTrace.traceln("ERROR: Caught Exception: " + e);
               System.out.println("\n--- Error returned from JGate flow ---\n");
               System.out.println("--- Caught Exception: " + e);
               break iteration_loop;
            }
         // end iEci_call for-loop
         }
      // end iIt_Count for-loop
      }
      // ---------------------------------------------------------------------
      // Close connection to JGate
      // ---------------------------------------------------------------------
      try
      {
         jgaConnection.close();
         System.out.println("\n--- Successfully closed JavaGateway connection ---");
      }
      catch (Exception e)
      {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         System.out.println("\n--- Error closing JavaGateway connection ---\n");
         System.out.println("--- Caught Exception: " + e);
      }
      System.out.println("\n--- Calls successful ... "+
                          +iReplyOK + " out of " + iCallsIssued );
      System.out.println("--- Iterations run ..... "+ (iIt_Count-1)
                          + " out of " + iIterations );
   }
}
