//Ŀ
//                                                                       
//  MODULE NAME      CA83.ZIP                                            
//                                                                       
//  DESCRIPTIVE NAME ECI Samples for the CICS Gateway for Java           
//                                                                       
//  Statement:       Licensed Materials - Property of IBM                
//                                                                       
//                   CA83 SupportPac                                     
//                   (c) Copyright IBM Corp. 1996, 1997                  
//                                                                       
//                   All rights reserved.                                
//                                                                       
//                   U.S. Government Users Restricted Rights - use,      
//                   duplication or disclosure restricted by GSA         
//                   ADP Schedule Contract with IBM Corp.                
//                                                                       
//  Status:          Version 1 Release 0 Modification 0                  
//                                                                       
//  Author:          Alan Hollingshead (hollings@hursley.ibm.com)        
//                                                                       
//   NOTES :-                                                            
//    DEPENDENCIES = JDK for chosen platform.                            
//                   CICS Gateway for Java on chosen platform.           
//                   CICS Client on same platform as Java Gateway.       
//                   Connection to CICS Server on any platform.          
//                   See CICS Gateway for Java doc for supported JDKs.   
//    RESTRICTIONS = None                                                
//                                                                       
//
//
// synj0c3 - Detect Maximum Commarea Length
//
// Written : Jun 97 - Alan Hollingshead
//
// This applet/application can issue non-extended ECI synchronous calls
// incrementing the commarea length until the maximum length (governed
// by CICS server and/or communication protocol) has been reached.
//
// Issuing "java synj0c3 -?" shows the following help and defaults:
//
//       SYNJ0C3 - Detect Maximum Commarea Length
//       Usage: java synj0c3 <parameters>
//              -j  JGate                      Default: (local hostname)
//              -r  JGate port                 Default: 2006
//              -s  CICS server                Default: (not specified)
//              -c  Initial commarea length    Default: 18
//              -+  increment commarea length  Default: false
//              -i  # iterations               Default: 1
//              -a  application                Default: SRVTIME
//              -u  userid                     Default: SYSAD
//              -p  password                   Default: SYSAD
//              -d  delay                      Default: 0 seconds
//              -!  debug                      Default: false
//              -?  This help panel
//
// Example: java synj0c3 -c 32000 -+
//
// The above example issues the first non-extended ECI synchronous call with
// commarea length 32000, and then further ECI calls incrementing the
// commarea length by one each time until an abend occurs (usually the
// maximum commarea size, plus one.).
// The commarea length is reset to the initial commarea length (-c) upon
// the start of each new iteration.
//
// The maximum commarea size varies according to the CICS server and
// communication protocol being used (ie SNA, NetBIOS, TCP/IP or named pipes).
//
// Initial commarea size should be 18 or more characters in length
// otherwise the commarea is not large enough to return a date and timestamp.
//
// Parameter overrides:
//      -j ... hostname or IP address where Java Gateway is running.
//             The default is your local hostname.
//      -r ... Java Gateway port. This defaults to 2006 unless
//             otherwise specified when starting the Java Gateway.
//      -s ... if the CICS Client is being used to connect to a CICS
//             server, null takes the first CICSCLI.INI setting.
//             Otherwise specify a valid entry defined in the CICSCLI.INI
//             file. If a CICS Client is not being used, a null value
//             will use the local server ECI, if supported.
//      -c ... initial commarea length. The maximum value that can be
//             entered is 65536 else a Java exception error will
//             (deliberately) appear.
//      -+ ... the commarea length is incremented by one each time until
//             an abend occurs. Usually this will be when the maximum
//             commarea size has been exceeded.
//      -i ... number of iterations (of each set of three calls).
//      -a ... name of server application.
//      -u ... userid, if required by the server mirror transaction.
//      -p ... password, if required by the server mirror transaction.
//      -d ... a delay in seconds between each iteration.
//      -! ... some debug info is supplied if "-!" is specified.
//      -? ... displays the help panel and defaults.
//
//
// If run as an applet - parameters passed via synj0c3.html
// If run as an application - parameters can be overridden via command line
// Issue "synj0c3 -?" for help and the default settings.
//
import ibm.cics.jgate.client.*;
import ibm.cics.jgate.server.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.applet.*;

public class synj0c3 extends Applet implements Runnable {
   // set up variables
   int iEci_call, iLUW, iCounter, iIt_Count;
   int iNum_LUWs      = 1;
   int iIterations    = 1;
   int iPort          = 2006;
   int iDelay         = 0;
   int iCallsIssued   = 0;
   int iReplyOK       = 0;
   int iComm_Len      = 18;
   String sDebug      = " ";
   String sInc_Comm   = " ";
   String sSpace      = "           ";

   // application defaults
   String sServer     = "" ;
   String sUseServer  = "" ;
   String sComm_Len   = "" ;
   String sUserid     = "SYSAD" ;
   String sPassword   = "SYSAD" ;
   String sProgname   = "SRVTIME" ;
   String sTransid    = "CPMI" ;
   boolean bDebug     = false ;
   boolean bIncrement_Comm = false ;
   String sJGate      = "" ;
   String sPort       = "2006" ;
   String sExt_mode   = "ECI_EXTENDED";
   String sNum_LUWs   = "1";
   String sIterations = "1";
   String sDelay      = "0";

   // Text area for program output
   TextArea texMessages;
   TextArea texSettings;

   // ------------------------------------------------------------------
   // Methods: displaySettings and displayMsg
   // When running as an applet ........ output sent to TextArea
   // When running as an application ... output sent to console
   // ------------------------------------------------------------------
   public void displaySettings(String strMessage) {
      if (texSettings != null) {
         texSettings.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }
   public void displayMsg(String strMessage) {
      if (texMessages != null) {
         texMessages.appendText(strMessage + "\n");
      }
      else {
         System.out.println(strMessage);
      }
   }
   public void displayCommarea(String strMessage) {
      if (texMessages != null) {
         texMessages.appendText(strMessage + "\r");
      }
      else {
         System.out.print(strMessage + "\r");
         System.out.flush();
      }
   }

   // ------------------------------------------------------------------
   // Method: init
   // Reads relevant parameters from html file when running as an applet
   // ------------------------------------------------------------------
   public void init() {
      // read applet parameters from html file
      sJGate      = getParameter("j");
      sPort       = getParameter("r");
      sServer     = getParameter("s");
      sComm_Len   = getParameter("c");
      iComm_Len   = Integer.parseInt(sComm_Len);
      sProgname   = getParameter("a");
      sIterations = getParameter("i");
      iIterations = Integer.parseInt(sIterations);
      sUserid     = getParameter("u");
      sPassword   = getParameter("p");
      sDelay      = getParameter("d");
      // convert to integer, and multiply to make seconds
      iDelay      = Integer.parseInt(sDelay) * 1000;
      sDebug      = getParameter("!");
      sInc_Comm   = getParameter("+");
      // convert to boolean
      if (sDebug.equals("true"))    bDebug = true;
      if (sInc_Comm.equals("true")) bIncrement_Comm = true;

      // Create a text area to display messages
      setLayout(null);
      Color lightYellow = new Color(240,240,204);
      setBackground(lightYellow);

      // add textareas
      texSettings = new TextArea();
      texMessages = new TextArea();
      add(texMessages);
      add(texSettings);

      // distance from left, top, then width, height
      texSettings.reshape(10, 130, 380, 260);
      texMessages.reshape(400, 30, 550, 400);

      // define font and colours
      texSettings.setFont(new Font("Courier", Font.PLAIN, 11));
      texSettings.setEditable(false);
      texSettings.setForeground(Color.darkGray);
      texSettings.setBackground(Color.white);

      texMessages.setFont(new Font("Courier", Font.PLAIN, 11));
      texMessages.setEditable(false);
      texMessages.setForeground(Color.blue);
      texMessages.setBackground(Color.white);

      Thread test = new Thread(this);
      this.show();
      this.run();
   }

   // ------------------------------------------------------------------
   // Method: paint
   // Display headings above the textarea
   // ------------------------------------------------------------------
   public void paint(Graphics g) {
      // simple heading
      g.setFont(new Font("Helvetica", Font.BOLD, 14));
      Color purple = new Color(39,25,204);
      g.setColor(purple);
      // distance from left, top
      g.drawString("SYNJ0C3", 10, 50);
      g.drawString("Detect Maximum Commarea Length", 10, 80);
   }

   // ------------------------------------------------------------------
   // Method: main
   // read in parameters when run as an application
   // ------------------------------------------------------------------
   static public void main(String[] args) {
      // declare and instantiate testApp object based on class synj0c3
      synj0c3 testApp = new synj0c3();

      // parse arguments entered on command line
      if (args.length > 0) {
         for (int loop=0 ; loop<args.length ; ++loop) {
            if ((args[loop].indexOf("-?") > -1) ||
                     (args[loop].indexOf("help") > -1)) {
               // HELP
               testApp.displaySettings("\nSYNJ0C3 - Detect Maximum Commarea Length") ;
               testApp.displaySettings("Usage: java synj0c3 <parameters>");
               testApp.displaySettings("       -j  JGate                      Default: (local hostname)");
               testApp.displaySettings("       -r  JGate port                 Default: "+testApp.sPort);
               testApp.displaySettings("       -s  CICS server                Default: (not specified)");
               testApp.displaySettings("       -c  Initial commarea length    Default: "+testApp.iComm_Len+" (Max: 65536)");
               testApp.displaySettings("       -+  increment commarea length  Default: "+testApp.bIncrement_Comm);
               testApp.displaySettings("       -i  # iterations               Default: "+testApp.sIterations);
               testApp.displaySettings("       -a  application                Default: "+testApp.sProgname);
               testApp.displaySettings("       -u  userid                     Default: "+testApp.sUserid);
               testApp.displaySettings("       -p  password                   Default: "+testApp.sPassword);
               testApp.displaySettings("       -d  delay between iterations   Default: "+testApp.sDelay+" seconds");
               testApp.displaySettings("       -!  debug                      Default: "+testApp.bDebug);
               testApp.displaySettings("       -?  This help panel");
               return;
            }
            else if (args[loop].indexOf("-!") > -1) {
               // set debug ON
               testApp.bDebug=true ;
               // for debug purposes
               GatewayTrace.setDebugOn(true);
               GatewayTrace.setOn(true);
            }
            else if (args[loop].indexOf("-+") > -1) {
               // set commarea increment counter
               testApp.bIncrement_Comm = true ;
            }
            else if (args[loop].indexOf("-s") > -1) {
               // set cics server
               testApp.sServer = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-j") > -1) {
               // set jgate server
               testApp.sJGate = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-r") > -1) {
               // set jgate port
               testApp.sPort = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iPort = Integer.parseInt(testApp.sPort);
            }
            else if (args[loop].indexOf("-u") > -1) {
               // set userid
               testApp.sUserid = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-p") > -1) {
               // set password
               testApp.sPassword = args[loop+1] ;
                ++loop ;
            }
            else if (args[loop].indexOf("-a") > -1) {
               // set progname
               testApp.sProgname = args[loop+1] ;
               ++loop ;
            }
            else if (args[loop].indexOf("-l") > -1) {
               // set Num_LUWs
               testApp.sNum_LUWs = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iNum_LUWs = Integer.parseInt(testApp.sNum_LUWs);
            }
            else if (args[loop].indexOf("-c") > -1) {
               // set length of commarea
               testApp.sComm_Len = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iComm_Len = Integer.parseInt(testApp.sComm_Len);
            }
            else if (args[loop].indexOf("-i") > -1) {
               // set Iterations
               testApp.sIterations = args[loop+1] ;
               ++loop ;
               // convert to integer
               testApp.iIterations = Integer.parseInt(testApp.sIterations);
            }
            else if (args[loop].indexOf("-d") > -1) {
               // set Delay between each ECI call
               testApp.sDelay = args[loop+1] ;
               ++loop ;
               // convert to integer, and multiply to make seconds
               testApp.iDelay = Integer.parseInt(testApp.sDelay) * 1000;
            }
         // end FOR loop
         }
      }
   // now go and do the work
   testApp.run();
   }

   // ------------------------------------------------------------------------
   // Method: run
   // Real work starts here
   // ------------------------------------------------------------------------
   public void run() {

      // if JGate not specified then use local hostname.
      // may require local hostname to be specified in hosts file.
      // eg on OS/2: \mptn\etc\hosts file.

      if (sJGate.equals("")) {
         try {
            sJGate = InetAddress.getLocalHost().getHostName();
         }
         catch (UnknownHostException e) {
            displaySettings("unresolved hostname - check hosts file or specify JGate");
            return;
         }
      }

      // check if server specified. If not, server ECI can be used if on
      // same machine as gateway, and server ECI module is earlier in LIBPATH
      // than a CICS Client one. Else use first server in Client CICSCLI.INI.
      if (sServer.equals("")) {
         sUseServer = "(not specified)";
      }
      else {
         sUseServer = sServer;
      }
      displaySettings("\nSYNJ0C3 - Detect Maximum Commarea Length") ;
      displaySettings("       -j  JGate                      -> "+sJGate);
      displaySettings("       -r  JGate port                 -> "+sPort);
      displaySettings("       -s  CICS server                -> "+sUseServer);
      displaySettings("       -c  initial commarea length    -> "+iComm_Len);
      displaySettings("       -+  increment commarea length  -> "+bIncrement_Comm);
      displaySettings("       -i  # iterations               -> "+sIterations);
      displaySettings("       -a  application                -> "+sProgname);
      displaySettings("       -u  userid                     -> "+sUserid);
      displaySettings("       -p  password                   -> "+sPassword);
      displaySettings("       -d  delay between iterations   -> "+sDelay+" seconds");
      displaySettings("       -!  debug                      -> "+bDebug);
      displaySettings("       -?  help panel");

      // create connection to JGate
      JavaGateway jgaConnection;
      jgaConnection=null;
      try {
         jgaConnection = new JavaGateway(sJGate, iPort);
         displayMsg("\n--- Successfully created JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Failed to create JavaGateway connection ---");
         displayMsg("--- Caught Exception: " + e);
         return;
      }

      // if server name has not been provided, get the server name
      if (sServer.equals("")) {
         ECIRequest eciRequest = ECIRequest.listSystems(1);
         try {
            jgaConnection.flow(eciRequest);
         }
         catch (Exception e) {
            displayMsg("\n--- ERROR during flow of listSystems(1) request ---");
            return;
         }
         // output info on the Java Vector
         displaySettings("\n--- Using CICS Region: "+ eciRequest.SystemList.elementAt(0));
      }

      // CICS commarea - assume max value to be 65536 (64K)
      // most CICS servers can handle <=32K (minus fmh bits)
      byte[] Commarea = new byte[65536];

      // ------------------------------------------------------------------
      // Extended Contructor ECIRequest()
      // ------------------------------------------------------------------

      ECIRequest eciRequest = new ECIRequest(ECIRequest.ECI_SYNC,
                                               sServer,
                                               sUserid,
                                               sPassword,
                                               sProgname,
                                               sTransid,
                                               Commarea,
                                               iComm_Len,
                                               0,
                                               0);

      String sGap1 = sSpace.substring(1,(12-(String.valueOf(iIterations).length())));
      displayMsg("\nIteration  Commarea Data (18)  Was/Is Length   CICS Rc   Abend");
      displayMsg("---------  ------------------  -------------   -------   -----");

      iteration_loop:
      for (iIt_Count=1; iIt_Count<=iIterations; iIt_Count++) {
         // set (or reset) initial commarea length
         eciRequest.Commarea_Length = iComm_Len;

         do {
            try {
                  // set commarea to dashes
                  for (iCounter=0; iCounter<eciRequest.Commarea_Length; iCounter++)
                     // reset commarea to dashes
                     eciRequest.Commarea[iCounter]=45;

                  // flow ECI request
                  iCallsIssued++;
                  jgaConnection.flow(eciRequest);

                  if (eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR) {
                     iReplyOK++;
                  }
                  else {
                     displayCommarea("\n");
                  }

                  // for debug purposes
                  GatewayTrace.traceln("Back again : AbendCode = " +
                                        eciRequest.Abend_Code);
                  GatewayTrace.traceln("eci RC " + eciRequest.Cics_Rc);

                  // Convert ASCII bytes to ASCII characters
                  String ComData  = new String(eciRequest.Commarea,0);

                  String sGap2 = sSpace.substring(1,(12-(String.valueOf(iIt_Count).length())));
                  String sGap3 = sSpace.substring(1,(9-(String.valueOf(iComm_Len).length())));
                  String sGap4 = sSpace.substring(1,(9-(String.valueOf(eciRequest.Commarea_Length).length())));
                  // Display result
                  displayCommarea(iIt_Count + sGap2 +
                       ComData.substring(0,18)+"  "+
                       iComm_Len + sGap3 +
                       eciRequest.Commarea_Length + sGap4 +
                       eciRequest.Cics_Rc +"        "+
                       eciRequest.Abend_Code +"    ");

               }
               catch (Exception e) {
                  GatewayTrace.traceln("ERROR: Caught Exception: " + e);
                  displayMsg("\n--- Error returned from JGate flow ---\n");
                  displayMsg("--- Caught Exception: " + e);
                  break iteration_loop;
               }
               eciRequest.Commarea_Length++;
           }

         while (bIncrement_Comm && eciRequest.Cics_Rc == ECIRequest.ECI_NO_ERROR);
      // start new line
      displayCommarea("\n");

      // Delay next iteration if delay parameter specified
      try {
         Thread.sleep(iDelay);
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Error during sleep() ---\n");
         displayMsg("--- Caught Exception: " + e);
         return;
      }
      // end iIt_Count for-loop
      }
      // ------------------------------------------------------------------
      // Close connection to JGate
      // ------------------------------------------------------------------
      try {
         jgaConnection.close();
         displayMsg("\n\n--- Successfully closed JavaGateway connection ---");
      }
      catch (Exception e) {
         GatewayTrace.traceln("ERROR: Caught Exception: " + e);
         displayMsg("\n--- Error closing JavaGateway connection ---\n");
         displayMsg("--- Caught Exception: " + e);
      }
      displaySettings("\n--- Calls successful ... "+
                         +iReplyOK + " out of " + iCallsIssued );
      displaySettings("--- Iterations run ..... "+ (iIt_Count-1)
                         + " out of " + iIterations );
   }
}
